#include "mainwindowimpl.h"

/*
	Constructor
*/
MainWindowImpl::MainWindowImpl( QWidget * parent, Qt::WFlags f) 
	: QMainWindow(parent, f)
{
	setupUi(this);
	
	// Load preferences
	readPrefs();
	
	QDir	cacheRootDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
	cacheRootDir.cdUp();
	if(!cacheRootDir.exists(APPNAME))
	{
		cacheRootDir.mkdir(APPNAME);
	}
	
	// Load abone file
	QFile	aboneFile(QDir::homePath() + "/" + ABONEFILENAME);
	if(aboneFile.open(QIODevice::ReadOnly))
	{
		ResParser::loadAboneFile(&aboneFile);
		aboneFile.close();
	}
	
	// Load favorites
	if(!QFile(QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "/" + FAVORITESFILENAME).exists())
	{
		writeFavorites();
	}
	readFavorites();
	
	// Load sessions
	readSession();
	if(prefs->windowWidth > 0)
	{
		this->resize(prefs->windowWidth, prefs->windowHeight);
	}

	// Timer initialize
	timerSec = 15;
	int	timerMin = 1;
	isAutoUpdating = false;
	reloadTimer.setInterval(timerSec * 1000);
	updateTimer.setSingleShot(true);
	updateTimer.setInterval(timerMin * 1000 * 60);
	
	// Reload button menu setup
	QMenu	*reloadMenu = new	QMenu();
	reloadStart = new QAction(QString(tr("Auto reload(interval : %1 sec)")).arg(timerSec), this);
	QAction	*intervalSetting = new QAction(tr("Auto reload interval setting..."), this);
	connect(reloadStart, SIGNAL(activated()), this, SLOT(startAutoReload()));
	connect(intervalSetting, SIGNAL(activated()), this, SLOT(showTimerDialog()));
	reloadMenu->addAction(reloadStart);
	reloadMenu->addAction(intervalSetting);
	reloadButton->setPopupMode(QToolButton::MenuButtonPopup);
	reloadButton->setMenu(reloadMenu);
	
	// Update button menu setup
	QMenu	*updateMenu = new	QMenu();
	updateStart = new QAction(QString(tr("Favorites auto update(interval : %1 min)")).arg(timerMin), this);
	QAction	*updateSetting = new QAction(tr("Favorites auto update interval..."), this);
	connect(updateStart, SIGNAL(activated()), this, SLOT(startAutoUpdate()));
	connect(updateSetting, SIGNAL(activated()), this, SLOT(showUpdateDialog()));
	updateMenu->addAction(updateStart);
	updateMenu->addAction(updateSetting);
	reloadSubjectButton->setPopupMode(QToolButton::MenuButtonPopup);
	reloadSubjectButton->setMenu(updateMenu);
	
	// Init dialogs
	prefsDialog = new Kimo2chPrefsDialog(this);
	prefsDialog->hide();
	timerDialog = new Kimo2chTimerDialog(this);
	timerDialog->hide();
	
	// Create models
	boardModel = new QStandardItemModel(0, 2);
	threadModel = new QStandardItemModel();
	filteredModel = new QStandardItemModel();
	
	hideAllPanes();
	network = new QHttp();
	postConnection = new QHttp();
	initialTab = true;
	
	// Connect signals and slots
	connect(prefsDialog, SIGNAL(accepted()), this, SLOT(prefsChange()));
	connect(actionRebuild_board_list, SIGNAL(triggered()), this, SLOT(rebuildBoardList()));
	connect(network, SIGNAL(requestFinished(int, bool)), this, SLOT(networkRequestFinished(int, bool)));
	connect(network, SIGNAL(responseHeaderReceived(const QHttpResponseHeader&)), this, SLOT(httpResponse(const QHttpResponseHeader&)));
	connect(network, SIGNAL(done(bool)), this, SLOT(upReloadButton()));
	connect(postConnection, SIGNAL(requestFinished(int, bool)), this, SLOT(postRequestFinished(int, bool)));
	connect(postConnection, SIGNAL(responseHeaderReceived(const QHttpResponseHeader&)), this, SLOT(postResponseHeader(const QHttpResponseHeader&)));
	connect(&reloadTimer, SIGNAL(timeout()), reloadButton, SLOT(click()));
	connect(&updateTimer, SIGNAL(timeout()), this, SLOT(updateFavoriteList()));
	QDialogButtonBox	*submitButton = (resEditTab->widget(0))->findChild<QDialogButtonBox*>("submitButton");
	connect(submitButton, SIGNAL(accepted()), this, SLOT(post()));
	connect(submitButton, SIGNAL(rejected()), this, SLOT(reject()));
	// Startup process
	QString boardURL = prefs->boardListURL;
	if(boardURL == "")
		boardURL = initialBoardListURL;
	if(1)
	{
		rebuildBoardList();
	}
//	isReadOnNewTab = prefsData->readOnNewTab;
	
	// Open favorites
	currentBoard = "Favorites";
	currentBoardName = tr("Favorites");
	loadSubject();
	// Restore previous session tabs
	QStringList::iterator	sessionIter = prefs->sessionURLs.begin();
	while(sessionIter != prefs->sessionURLs.end())
	{
		openTab(*sessionIter);
		sessionIter++;
	}
}

/*
	Destructor
*/
MainWindowImpl::~MainWindowImpl()
{
	// Record session to file
	writePrefs();
	writeSession();
	writeFavorites();
	QFile	aboneFile(QDir::homePath() + "/" + ABONEFILENAME);
	ResParser::saveAboneFile(&aboneFile);
}

/***************************
	Preferences management
***************************/
/*
	Save prefs
*/
void	MainWindowImpl::writePrefs()
{
	
}

/*
	Load prefs
*/
void	MainWindowImpl::readPrefs()
{
	
}

/***************************
	Favorites management
***************************/

void	MainWindowImpl::writeFavorites()
{
	QFile	favoritesFile(QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "/" + FAVORITESFILENAME);
	if(!favoritesFile.open(QIODevice::WriteOnly))
	{
		QMessageBox::warning(this, tr("File Error"), tr("Cannot write to favorites file"));
		return;
	}
	QXmlStreamWriter	favoritesWriter(&favoritesFile);
	favoritesWriter.setAutoFormatting(true);
	favoritesWriter.writeStartDocument();
	favoritesWriter.writeStartElement("", "favorites");
//	if(favoriteList
	QList<Favorite>::iterator	iter = favoriteList.begin();
	while(iter != favoriteList.end())
	{
		favoritesWriter.writeStartElement("", "favoriteItem");
		favoritesWriter.writeStartElement("", "name");
		favoritesWriter.writeCharacters((*iter).name);
		favoritesWriter.writeEndElement();
		favoritesWriter.writeStartElement("", "URL");
		favoritesWriter.writeCharacters((*iter).URL);
		favoritesWriter.writeEndElement();
		favoritesWriter.writeEndElement();
		iter++;
	}
	favoritesWriter.writeEndElement();
	favoritesWriter.writeEndDocument();
	favoritesFile.close();
}

void	MainWindowImpl::readFavorites()
{
//	favoriteList = new QList<Favorite>();
	QFile	favoritesFile(QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "/" + FAVORITESFILENAME);
	if(favoritesFile.exists())
	{
		if(!favoritesFile.open(QIODevice::ReadOnly))
		{
			QMessageBox::warning(this, tr("File Error"), tr("Cannot read favorites file."));
			return;
		}
		QXmlInputSource	in(&favoritesFile);
		QXmlSimpleReader	reader;
		Kimo2chFavoritesHandler	handler(&favoriteList);
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
		reader.parse(&in);
		favoritesFile.close();
	}
}

/***************************
	Session management
***************************/
/*
	Save session
*/
void	MainWindowImpl::writeSession()
{
	QDir	dataDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
	if(!dataDir.exists())
	{
		dataDir.cdUp();
		if(!dataDir.mkdir(APPNAME))
		{
			QMessageBox::warning(this, tr("File Error"), tr("Can't make data directory."));
			return;
		}
		dataDir.cd(APPNAME);
	}
	QFile	prefsFile(dataDir.filePath(APPNAME + "rc"));
//	QFile	prefsFile(QDir::homePath() + "/." + APPNAME + "rc");
	if(!prefsFile.open(QIODevice::WriteOnly))
	{
		QMessageBox::information(this, tr("Error"), tr("Preference file save failure."));
		return;
	}
	// Write XML document to file
	QXmlStreamWriter	writer(&prefsFile);
	writer.setAutoFormatting(true);
	writer.writeStartDocument();
	writer.writeStartElement("", "Prefs");
	writer.writeStartElement("", "AppPreferences");
		writer.writeStartElement("", "browserFontFamily");
		writer.writeCharacters(prefs->browserFontFamily);
		writer.writeEndElement();
		writer.writeStartElement("", "browserFontSize");
		writer.writeCharacters(QString("%1").arg(prefs->browserFontSize));
		writer.writeEndElement();
		writer.writeStartElement("", "initialName");
		writer.writeCharacters(prefs->initialName);
		writer.writeEndElement();
		writer.writeStartElement("", "initialMail");
		writer.writeCharacters(prefs->initialMail);
		writer.writeEndElement();
		writer.writeStartElement("", "defaultSage");
		writer.writeCharacters((prefs->defaultSage) ? "true" : "false");
		writer.writeEndElement();
	writer.writeEndElement();
	writer.writeStartElement("", "Sessions");
		// Record window size
		writer.writeStartElement("", "windowWidth");
		writer.writeCharacters(QString("%1").arg(width()));
		writer.writeEndElement();
		writer.writeStartElement("", "windowHeight");
		writer.writeCharacters(QString("%1").arg(height()));
		writer.writeEndElement();
		// Record subject list columns size
		for(int	i = 0; i < 4; i++)
		{
			writer.writeStartElement("", "columnWidth");
			qDebug() << " ColumnWidth=" << threadList->columnWidth(i) << endl;
			writer.writeCharacters(threadList->columnWidth(i) > 0 ? QString::number(threadList->columnWidth(i)) : 
				QString::number(prefs->columnWidth[i]));
			writer.writeEndElement();
		}
		// Record opened browser tabs
		int	tabCount = browserTab->count();
		for(int	i = 0; i < tabCount; i++)
		{
			BrowseWidgetImpl	*widget = dynamic_cast<BrowseWidgetImpl*>(browserTab->widget(i));
			if(widget)
			{
				writer.writeStartElement("", "sessionURL");
				writer.writeCharacters(widget->getUrl());
				writer.writeEndElement();
			}
		}
	writer.writeEndElement();
	writer.writeEndElement();
	writer.writeEndDocument();
}
/*
	Load session
*/
void	MainWindowImpl::readSession()
{
	prefs = new Kimo2chPrefs;
	QDir	dataDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
	QFile	prefsFile;
	if(dataDir.exists())
	{
		qDebug() << dataDir.path();
		prefsFile.setFileName(dataDir.filePath(APPNAME + "rc"));
	}
	else
	{
		return;
	}
	if(prefsFile.exists())
	{
		if(!prefsFile.open(QIODevice::ReadOnly))
		{
			QMessageBox::information(this, tr("IOError"), tr("Cannnot read prefs file."));
			return;
		}
		QXmlInputSource	in(&prefsFile);
		XmlPrefsHandler	handler(prefs);
		QXmlSimpleReader	reader;
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
		reader.parse(&in);
	}
	prefsFile.close();
}
/**************************
	View control
**************************/
/*
	Show / hide views
*/
void MainWindowImpl::showBoardList()
{
	boardList->setMaximumWidth(32767);
}

void MainWindowImpl::showThreadList()
{
	threadFrame->setMaximumHeight(32767);
}

void MainWindowImpl::showResEditTab()
{
	resEditTab->setMaximumHeight(32767);
	reloadTimer.stop();
}

void MainWindowImpl::hideAllPanes()
{
	boardList->setMaximumWidth(0);
	threadFrame->setMaximumHeight(0);
	resEditTab->setMaximumHeight(0);
	if(isAutoReloading && (!reloadTimer.isActive()))
	{
		reloadTimer.start();
	}
}

void MainWindowImpl::rememberColumnWidth()
{
	if(threadList->model() != NULL)
	{
		int	colNum = threadList->model()->columnCount();
		for(int	col = 0; col < colNum; col++)
		{
			int	width;
			if((width = threadList->columnWidth(col)) > 0)
				prefs->columnWidth[col] = width;
		}
	}
}

void MainWindowImpl::restoreColumnWidth()
{
	if(threadList->model() != NULL)
	{
		int	colNum = threadList->model()->columnCount();
		for(int	col = 0; col < colNum; col++)
		{
			threadList->setColumnWidth(col, prefs->columnWidth[col]);
		}
	}
}
/*
 * Hide popups at switch tab
 */

void	MainWindowImpl::hidePopups()
{
	for(int	i = 0; i <= browserTab->count(); i++)
	{
		BrowseWidgetImpl	*tab = dynamic_cast<BrowseWidgetImpl*>(browserTab->widget(i));
		if(tab)
		{
			Kimo2chTextBrowser	*browser = tab->findChild<Kimo2chTextBrowser*>("resBrowser");
			if(browser)
			{
				browser->closePopup();
			}
		}
	}
}

void	MainWindowImpl::filterLabelChange()
{
	filterLabel->setText(tr("Filter"));
	if(filterEdit->text() == currentBoardName)
	{
		filterEdit->clear();
	}
}

void	MainWindowImpl::filterLabelRevert()
{
	if(filterEdit->text().isEmpty())
	{
		filterLabel->setText(tr("Board"));
		filterEdit->setText(currentBoardName);
	}
}

void	MainWindowImpl::searchLabelChange()
{
	searchLabel->setText(tr("Search"));
	if(searchEdit->text() == browserTab->tabText(browserTab->currentIndex()))
	{
		searchEdit->clear();
		searchEdit->setFocus();
	}
}

void	MainWindowImpl::searchLabelRevert()
{
	if(searchEdit->text().isEmpty())
	{
		searchLabel->setText(tr("Thread"));
		BrowseWidgetImpl	*currentBrowser = dynamic_cast<BrowseWidgetImpl*>(browserTab->currentWidget());
		if(currentBrowser != NULL)
		{
			searchEdit->setText(currentBrowser->getTitle());
		}
//		searchEdit->setText(browserTab->tabText(browserTab->cur currentIndex()));
	}
	searchEdit->clearFocus();
}
#if 0
void	MainWindowImpl::searchLabelReset()
{
	searchEdit->clear();
	searchLabelRevert();
}
#endif
void	MainWindowImpl::downReloadButton()
{
	reloadButton->setDown(true);
}

void	MainWindowImpl::upReloadButton()
{
	reloadButton->setDisabled(false);
	reloadButton->setDown(false);
}

/**************************
	Networking
**************************/

/*
	Rebuild board(BBS) list
*/
void MainWindowImpl::rebuildBoardList()
{
	boardBuf = new QBuffer();
	boardBuf->open(QIODevice::ReadWrite);
	QUrl url(initialBoardListURL);
	network->setHost(url.host());
	boardListGetID = network->get(url.path(), boardBuf);
}

/*
	Subject selected in board(BBS) list
*/
void MainWindowImpl::boardSelected(QModelIndex index)
{
	if(index.sibling(index.row(), 1).data().toString() == "")
	{
		boardList->setExpanded(index, true);
		return;
	}
	currentBoard = index.sibling(index.row(), 1).data().toString();
	currentBoardName = index.sibling(index.row(), 0).data().toString();
	filterLabelRevert();
	filterEdit->setText(currentBoardName);
	if(currentBoard != "Favorites")
	{
		startAutoUpdate(false);
	}
	loadSubject();
}

/*
	Download subject.txt
*/
void MainWindowImpl::loadSubject()
{
//	threadModelMutex.lock();
	if(isFavoritesUpdating)
	{
		if(network->state() != QHttp::Unconnected)
		{
			network->abort();
		}
		isFavoritesUpdating = false;
	}

	if(currentBoard.isEmpty())
	{
		QMessageBox::information(this, "info", currentBoardName);
		return;
	}
	reloadSubjectButton->setDown(true);
	reloadSubjectButton->setEnabled(false);
	rememberColumnWidth();
//	subjectBufMutex.lock();
	subjectBuf = new QBuffer();
	subjectBuf->open(QIODevice::ReadWrite);
	QHttpRequestHeader header;
	header.setValue("User-Agent", QString("Monazilla/1.00 (%1)").arg(APPNAME));
	
	if(currentBoard == "Favorites")
	{
		// Favorites unread count reset.
		targetBoardList.clear();
		unreadThreadNum = unreadResNum = 0;
		isFavoritesUpdating = true;
		threadModel = new QStandardItemModel();
		threadModel->clear();
		if(favoriteList.isEmpty())
		{
			threadList->setModel(threadModel);
			return;
		}
		QString	url;
		QStringList	urlItem;
		
		// Assort favorites by board
		QList<QList<Favorite>* >::iterator	targetIter;
		QList<Favorite>::iterator	favoriteIter = favoriteList.begin();
		while(favoriteIter != favoriteList.end())
		{
			QString	targetURL = (*favoriteIter).URL.mid(0, (*favoriteIter).URL.lastIndexOf('/', (*favoriteIter).URL.lastIndexOf('/') - 1));
			bool	added = false;
			targetIter = targetBoardList.begin();
			while(targetIter != targetBoardList.end())
			{
				if((*targetIter)->at(0).URL.contains(targetURL))
				{
					(*targetIter)->append(*favoriteIter);
					added = true;
				}
				targetIter++;
			}
			if(!added)
			{
				QList<Favorite>	*newBoard = new	QList<Favorite>();
				newBoard->append(*favoriteIter);
				targetBoardList.append(newBoard);
			}
			favoriteIter++;
		}
		
		// Download favorites subject.txt
		QString	urlText = targetBoardList.first()->first().URL;
		qDebug() <<urlText;
		QUrl	boardUrl(urlText.mid(0, urlText.lastIndexOf('/', urlText.lastIndexOf('/') - 1) + 1) + "subject.txt");
		qDebug() << boardUrl.host() << boardUrl.path();
		network->setHost(boardUrl.host());
		header.setRequest("GET", boardUrl.path());
		header.addValue("Host", boardUrl.host());
		favoriteUpdateID = network->request(header, "", subjectBuf);
		return;
		
		targetIter = targetBoardList.begin();
		while(targetIter != targetBoardList.end())
		{
			// Download target board "subject.txt"
			QString	tmpurl = (*targetIter)->at(0).URL;
			QString	subjectURL = tmpurl.mid(0, tmpurl.lastIndexOf('/', tmpurl.lastIndexOf('/'))) + "subject.txt";
			
		}
	}
	QUrl url(currentBoard + "subject.txt");
	network->setHost(url.host());
	header.setRequest("GET", url.path());
	header.addValue("Host", url.host());
	subjectGetID = network->request(header, "", subjectBuf);
}

/*
	Thread reloading action
*/
void MainWindowImpl::readThread(QModelIndex index)
{
	QString thread = index.sibling(index.row(), urlCol).data().toString();
	openTab(thread);
}

 void	MainWindowImpl::openTab(QString	url)
{
	// Add favorite button reset
	addFavoriteButton->setDown(false);
	
	int	tabCount = browserTab->count();
	for(int	i = 0; i < tabCount; i++)
	{
		BrowseWidgetImpl	*tab = dynamic_cast<BrowseWidgetImpl*>(browserTab->widget(i));
		if(tab)
		{
			if(tab->getThreadUrl() == url)
			{
				browserTab->setCurrentIndex(i);
				if(tab->isFavorite())
				{
					addFavoriteButton->setDown(true);
				}
				tab->reload();
				return;
			}
		}
	}
	BrowseWidgetImpl	*newTab = new BrowseWidgetImpl(url, prefs->browserFontFamily, prefs->browserFontSize, browserTab);
	if(Kimo2chTextBrowser	*browser = newTab->findChild<Kimo2chTextBrowser*>("resBrowser"))
	{
		connect(browser, SIGNAL(linkClicked(const QUrl&)), this, SLOT(openLink(const QUrl&)));
		connect(newTab, SIGNAL(reloadCompleted()), this, SLOT(upReloadButton()));
	}
	browserTab->addTab(newTab, newTab->getTitle());
	QList<Favorite>::iterator	iter = favoriteList.begin();
	while(iter != favoriteList.end())
	{
		if((*iter).URL == url)
		{
			newTab->setFavorite(true);
			addFavoriteButton->setDown(true);
		}
		iter++;
	}
	newTab->reload();
	browserTab->setCurrentIndex(browserTab->count() - 1);
	if(initialTab)
	{
		browserTab->removeTab(0);
		initialTab = false;
	}
	writeSession();
}

/*
	Current tab changed
*/
void MainWindowImpl::threadTabChanged(int index)
{
	startAutoReload(false);
	searchEdit->clear();
	searchLabelRevert();
//	searchEdit->setText(browserTab->tabText(browserTab->currentIndex()));
	searchEdit->clearFocus();
	addFavoriteButton->setDown(false);
	
	// Tab switching
	BrowseWidgetImpl	*tab = dynamic_cast<BrowseWidgetImpl*>(browserTab->widget(index));
	QString	urlText;
	// Set state favorite button
	if(tab)
	{
		urlText = tab->getUrl();
		searchEdit->setText(tab->getTitle());
		if(tab->isFavorite())
		{
			addFavoriteButton->setDown(true);
		}
	}
	// Switch editor tab follows browser tab URL
	int	editorTabCount = resEditTab->count();
	for(int	i = 1; i < editorTabCount; i++)
	{
		if(resEditTab && resEditTab->widget(i)->findChild<QLabel*>("editTitleLabel")->text() == tab->getUrl())
		{
			resEditTab->setCurrentIndex(i);
			return;
		}
	}
	qDebug() << "done!";
	// If first browser tab unchanged
	if(/*firstTab && tab && */resEditTab->widget(0)->findChild<QPlainTextEdit*>("resEdit")->toPlainText().isEmpty()
		&& resEditTab->widget(0)->findChild<QLineEdit*>("nameForm")->text().isEmpty()
		&& (resEditTab->widget(0)->findChild<QLineEdit*>("mailForm")->text().isEmpty() || resEditTab->widget(0)->findChild<QLineEdit*>("mailForm")->text() == prefs->initialMail))
	{
		resEditTab->widget(0)->findChild<QLabel*>("editTitleLabel")->setText(urlText);
		resEditTab->setTabText(0, tab ? tab->getTitle() : "");
		resEditTab->setCurrentIndex(0);
		return;
	}
	else
	{
		QWidget	*newTab = new TabWidgetImpl(resEditTab);
		resEditTab->insertTab(0, newTab, tab->getTitle());
		newTab->findChild<QLabel*>("editTitleLabel")->setText(urlText);
		newTab->findChild<QLineEdit*>("mailForm")->setText(prefs->initialMail);
		resEditTab->setCurrentIndex(0);
		QDialogButtonBox	*submitButton = newTab->findChild<QDialogButtonBox*>("submitButton");
		connect(submitButton, SIGNAL(accepted()), this, SLOT(post()));
		connect(submitButton, SIGNAL(rejected()), this, SLOT(reject()));
	}
}

/*
	Reload tab
*/
void	MainWindowImpl::reload()
{
	BrowseWidgetImpl	*tab = dynamic_cast<BrowseWidgetImpl*>(browserTab->currentWidget());
	if(tab)
	{
		reloadButton->setDisabled(true);
		reloadButton->setDown(true);
		tab->reload();
	}
}
/*
	Close tab
*/
void	MainWindowImpl::closeTab(int	index)
{
	QWidget	*target = browserTab->widget(index);
	if(target != NULL)
	{
		delete	target;
		target = NULL;
	}
}

void	MainWindowImpl::findNextThreads()
{
	BrowseWidgetImpl	*tab = dynamic_cast<BrowseWidgetImpl*>(browserTab->currentWidget());
	if(tab)
	{
		tab->downloadSubjectForNextThread();
	}
}

/*******************************
	Preferences Panel
*******************************/
/*
	Show preferences panel
*/
void MainWindowImpl::showPrefsPanel()
{
	prefsDialog->findChild<QLineEdit*>("boardURL")->setText(prefs->boardListURL);
//	prefsDialog->findChild<QCheckBox*>("rebuildCheck")->setCheckState(prefsData->rebuildBoardList);
	prefsDialog->findChild<QFontComboBox*>("browserFontBox")->setCurrentFont(QFont(prefs->browserFontFamily));
	prefsDialog->findChild<QSpinBox*>("browserFontSizeBox")->setValue(prefs->browserFontSize);
//	prefsDialog->findChild<QCheckBox*>("newTabCheck")->setCheckState(prefsData->readOnNewTab);
	prefsDialog->findChild<QLineEdit*>("name")->setText(prefs->initialName);
	prefsDialog->findChild<QLineEdit*>("mail")->setText(prefs->initialMail);
	prefsDialog->findChild<QCheckBox*>("sageCheck")->setCheckState(prefs->defaultSage ? Qt::Checked : Qt::Unchecked);
	prefsDialog->show();
	prefsDialog->exec();
}

/*
	Change application preferences
*/
void MainWindowImpl::prefsChange()
{
	prefs->boardListURL = prefsDialog->findChild<QLineEdit*>("boardURL")->text();
//	prefsData->rebuildBoardList = prefsDialog->findChild<QCheckBox*>("rebuildCheck")->checkState();
	prefs->browserFontFamily = prefsDialog->findChild<QFontComboBox*>("browserFontBox")->currentFont().family();
	prefs->browserFontSize = prefsDialog->findChild<QSpinBox*>("browserFontSizeBox")->value();
//	prefsData->readOnNewTab = prefsDialog->findChild<QCheckBox*>("newTabCheck")->checkState();
	prefs->initialName = prefsDialog->findChild<QLineEdit*>("name")->text();
	prefs->initialMail = prefsDialog->findChild<QLineEdit*>("mail")->text();
	prefs->defaultSage = prefsDialog->findChild<QCheckBox*>("sageCheck")->checkState();
	prefsDialog->hide();
}

/*****************************
	Abone dialog
*****************************/

// Show abone list dialog

void MainWindowImpl::showAboneDialog()
{
	aboneDialog = new AboneDialog();
	
	QListView	*aboneIdView = aboneDialog->findChild<QListView*>("aboneIdView");
	QListView	*aboneNameView = aboneDialog->findChild<QListView*>("aboneNameView");
	QListView	*aboneMailView = aboneDialog->findChild<QListView*>("aboneMailView");
	QListView	*aboneWordView = aboneDialog->findChild<QListView*>("aboneWordView");
	if(aboneIdView == NULL)
	{
		qDebug() << "aboneIdView is NULL!";
		return;
	}
	// Copy abone lists to temporaries
	QStandardItemModel	*aboneIdModel = new QStandardItemModel;
	QStandardItemModel	*aboneNameModel = new QStandardItemModel;
	QStandardItemModel	*aboneMailModel = new QStandardItemModel;
	QStandardItemModel	*aboneWordModel = new QStandardItemModel;

	// Abone list contents setup
	QMap<QString, bool>::const_iterator	iter;
	for(iter = ResParser::aboneId.constBegin(); iter != ResParser::aboneId.constEnd(); iter++)
	{
		QStandardItem	*aboneIdItem = new	QStandardItem(iter.key());
		aboneIdModel->appendRow(aboneIdItem);
	}
	aboneIdView->setModel(aboneIdModel);
	for(iter = ResParser::aboneName.constBegin(); iter != ResParser::aboneName.constEnd(); iter++)
	{
		QStandardItem	*aboneNameItem = new	QStandardItem(iter.key());
		aboneNameModel->appendRow(aboneNameItem);
	}
	aboneNameView->setModel(aboneNameModel);
	for(iter = ResParser::aboneMail.constBegin(); iter != ResParser::aboneMail.constEnd(); iter++)
	{
		QStandardItem	*aboneMailItem = new	QStandardItem(iter.key());
		aboneMailModel->appendRow(aboneMailItem);
	}
	aboneMailView->setModel(aboneMailModel);
	for(iter = ResParser::aboneWord.constBegin(); iter != ResParser::aboneWord.constEnd(); iter++)
	{
		QStandardItem	*aboneWordItem = new	QStandardItem(iter.key());
		aboneWordModel->appendRow(aboneWordItem);
	}
	aboneWordView->setModel(aboneWordModel);
	
	QDialogButtonBox	*aboneButtonBox = aboneDialog->findChild<QDialogButtonBox*>("aboneButtonBox");
	connect(aboneButtonBox, SIGNAL(accepted()), this, SLOT(aboneChange()));
	connect(aboneButtonBox, SIGNAL(rejected()), this, SLOT(closeAboneDialog()));
	aboneDialog->show();
	aboneDialog->exec();
}

// Change abone lists

void MainWindowImpl::aboneChange()
{
	if(aboneDialog)
	{
		QStandardItemModel	*aboneIdModel = 
			dynamic_cast<QStandardItemModel*>(aboneDialog->findChild<QListView*>("aboneIdView")->model());
		QStandardItemModel	*aboneNameModel =
			dynamic_cast<QStandardItemModel*>(aboneDialog->findChild<QListView*>("aboneNameView")->model());
		QStandardItemModel	*aboneMailModel = 
			dynamic_cast<QStandardItemModel*>(aboneDialog->findChild<QListView*>("aboneMailView")->model());
		QStandardItemModel	*aboneWordModel = 
			dynamic_cast<QStandardItemModel*>(aboneDialog->findChild<QListView*>("aboneWordView")->model());
		if(aboneIdModel != NULL)
		{
			ResParser::aboneId.clear();
			QList<QStandardItem*>	aboneIdList = aboneIdModel->takeColumn(0);
			QList<QStandardItem*>::iterator	iter = aboneIdList.begin();
			while(iter != aboneIdList.end())
			{
				ResParser::aboneId.insert((*iter)->text(), false);
				iter++;
			}
		}
		if(aboneNameModel != NULL)
		{
			ResParser::aboneName.clear();
			QList<QStandardItem*>	aboneNameList = aboneNameModel->takeColumn(0);
			QList<QStandardItem*>::iterator iter = aboneNameList.begin();
			while(iter != aboneNameList.end())
			{
				ResParser::aboneName.insert((*iter)->text(), false);
				iter++;
			}
		}
		if(aboneMailModel != NULL)
		{
			ResParser::aboneMail.clear();
			QList<QStandardItem*>	aboneMailList = aboneMailModel->takeColumn(0);
			QList<QStandardItem*>::iterator iter = aboneMailList.begin();
			while(iter != aboneMailList.end())
			{
				ResParser::aboneMail.insert((*iter)->text(), false);
				iter++;
			}
		}
		if(aboneWordModel != NULL)
		{
			ResParser::aboneWord.clear();
			QList<QStandardItem*>	aboneWordList = aboneWordModel->takeColumn(0);
			QList<QStandardItem*>::iterator iter = aboneWordList.begin();
			while(iter != aboneWordList.end())
			{
				ResParser::aboneWord.insert((*iter)->text(), false);
				iter++;
			}
		}
		closeAboneDialog();
	}
}

// Revert before edit abone lists

void MainWindowImpl::closeAboneDialog()
{
	aboneDialog->close();
	delete	aboneDialog;
	aboneDialog = NULL;
}
/*****************************
	Auto reload
*****************************/

void	MainWindowImpl::startAutoReload(bool	state)
{
	BrowseWidgetImpl*	currentBrowser = dynamic_cast<BrowseWidgetImpl*>(browserTab->currentWidget());
	if(state)
	{
		if(!reloadTimer.isActive())
		{
			reload();
			reloadTimer.start();
			isAutoReloading = true;
			reloadStart->setText(tr("Stop auto reload"));
			if(currentBrowser)
				currentBrowser->setAutoReload(true);
			return;
		}
	}
	reloadTimer.stop();
	isAutoReloading = false;
	reloadStart->setText(QString(tr("Auto reload (interval %1 sec)")).arg(reloadTimer.interval() / 1000));
	if(currentBrowser)
		currentBrowser->setAutoReload(false);
}

void	MainWindowImpl::showTimerDialog()
{
	Kimo2chTimerDialog	reloadTimerDialog(this);
	QSpinBox	*intervalBox = reloadTimerDialog.findChild<QSpinBox*>("intervalBox");
	if(intervalBox != NULL)
	{
		intervalBox->setValue(reloadTimer.interval() / 1000);
	}
	int	result = reloadTimerDialog.exec();
	if(result == QDialog::Accepted)
	{
		timerSec = intervalBox->value();
		reloadTimer.setInterval(intervalBox->value() * 1000);
		if(!reloadTimer.isActive())
		{
			startAutoReload(false);
		}
	}
	reloadTimerDialog.close();
	return;
	timerDialog->findChild<QSpinBox*>("intervalBox")->setValue(timerSec);
	timerDialog->show();
	timerDialog->exec();
}

void	MainWindowImpl::timerChange()
{
	timerSec = timerDialog->findChild<QSpinBox*>("intervalBox")->value();
	reloadTimer.setInterval(timerSec * 1000);
	reloadStart->setText(QString(tr("Auto reload start %1 sec interval")).arg(timerSec));
	timerDialog->hide();
}

/*
 * Favorite list auto update
 */

void	MainWindowImpl::startAutoUpdate(bool	state)
{
	if(state)
	{
		if(!isAutoUpdating)
		{
			updateFavoriteList();
			isAutoUpdating = true;
			updateTimer.start();
			updateStart->setText(tr("Stop auto update"));
			statusBar()->showMessage(tr("Auto update start."), 10 * 1000);
			return;
		}
	}
	isAutoUpdating = false;
	updateTimer.stop();
	updateStart->setText(QString(tr("Favorites auto update start (interval : %1 min)")).arg(updateTimer.interval() / 1000 / 60));
	statusBar()->showMessage(tr("Auto update stopped."));
}

void	MainWindowImpl::showUpdateDialog()
{
	Kimo2chTimerDialog	updateTimerDialog(this);
	QLabel	*infoLabel = updateTimerDialog.findChild<QLabel*>("infoLabel");
	if(infoLabel != NULL)
	{
		infoLabel->setText(tr("Update interval (min)"));
	}
	QSpinBox	*intervalBox = updateTimerDialog.findChild<QSpinBox*>("intervalBox");
	if(intervalBox != NULL)
	{
		intervalBox->setValue(updateTimer.interval() / 1000 / 60);
	}
	if(updateTimerDialog.exec() == QDialog::Accepted)
	{
		updateTimer.setInterval(intervalBox->value() * 1000 * 60);
	}
	updateTimerDialog.close();
}

/*****************************
	Search dialog
*****************************/

void	MainWindowImpl::doSearch(QString	searchText)
{
	if(searchText == 0)
	{
		searchText = searchEdit->text();
	}
	BrowseWidgetImpl		*browser = dynamic_cast<BrowseWidgetImpl*>(browserTab->currentWidget());
	if(browser)
	{
		browser->findChild<QTextBrowser*>("resBrowser")->find(searchText);
	}
}

void	MainWindowImpl::doFilter(QString	filterText)
{
//	QString	filterText = filterEdit->text();
	if(threadModel && !(filterText.isEmpty()))
	{
		rememberColumnWidth();
		QList<QStandardItem*>	result = threadModel->findItems(filterText, Qt::MatchContains, subjectCol);
		filteredModel->clear();
		QStringList	header;
		header << tr("URL") << tr("Title") << tr("Count") << tr("New");
		filteredModel->setHorizontalHeaderLabels(header);
		QModelIndex	index;
		int	row = 0;
		QList<QStandardItem*>::iterator	iter(result.begin());
		while(iter != result.end())
		{
			index = threadModel->indexFromItem(*iter);
			filteredModel->setItem(row, urlCol, new	QStandardItem(threadModel->item(index.row(), urlCol)->text()));
			filteredModel->setItem(row, subjectCol, new	QStandardItem((*iter)->text()));
			filteredModel->setItem(row, countCol, new	ResCountItem(threadModel->item(index.row(), countCol)->text()));
			filteredModel->setItem(row, newCol, new	ResCountItem(threadModel->item(index.row(), newCol)->text()));
			iter++, row++;
		}
		threadList->setModel(filteredModel);
	}
	else
	{
		threadList->setModel(threadModel);
	}
	restoreColumnWidth();
	threadList->hideColumn(urlCol);
	threadList->resizeRowsToContents();
	
}

/*****************************
	Favorite actions
*****************************/
/*
 * addFavoriteButton pushed
 */
 
void	MainWindowImpl::toggleAddFavoriteButton()
{
	BrowseWidgetImpl	*browseWidget = dynamic_cast<BrowseWidgetImpl*>(browserTab->currentWidget());
	if(browseWidget == NULL)
	{
		return;
	}
	QString	targetUrl = browseWidget->getUrl();
	// Remove current tab item from favorites.
	if(browseWidget->isFavorite())
	{
		QList<Favorite>::iterator	favIter = favoriteList.begin();
		while(favIter != favoriteList.end())
		{
			if((*favIter).URL == targetUrl)
			{
				favoriteList.erase(favIter);
				break;
			}
			favIter++;
		}
		threadModel->takeRow(threadModel->indexFromItem(threadModel->findItems(targetUrl).at(0)).row());
		browseWidget->setFavorite(false);
		addFavoriteButton->setDown(false);
		writeFavorites();
		return;
	}
	else
	// Add current to favorites.
	{
		Favorite	newFavorite;
		newFavorite.URL = targetUrl;
		newFavorite.name = browseWidget->getTitle();
		newFavorite.resCount = 0;
		favoriteList.append(newFavorite);
		QList<QStandardItem*>	newItems;
		newItems << new	QStandardItem(targetUrl) << new	QStandardItem(browseWidget->getTitle());
		threadModel->insertRow(0, newItems);
		threadList->resizeRowToContents(0);
		browseWidget->setFavorite(true);
		addFavoriteButton->setDown(true);
		writeFavorites();
	}
}

/*****************************
	Networking
*****************************/

/*
	Open link when anchor clicked
*/
void	MainWindowImpl::openLink(const QUrl	&url)
{
	bool	autoReloadNextThread = reloadTimer.isActive();
	if(url.path().contains("test/read.cgi"))
	{
		QString	threadUrl = url.toString();
		QStringList	urlItems;
		if(threadUrl.contains('/'))
		{
			urlItems = threadUrl.split('/');
		}
		if(urlItems.count() > 6)
		{
			QString	targetUrl = QString("http://%1/%2/dat/%3.dat").arg(urlItems[2], urlItems[5], urlItems[6]);
			openTab(targetUrl);
			return;
		}
	}
	if(url.path().contains("/dat/") && url.path().right(4) == ".dat")
	{
		openTab(url.toString());
		startAutoReload(autoReloadNextThread);
		return;
	}
	QDesktopServices::openUrl(url);
}

/*
	Process HTTP response
*/
void MainWindowImpl::networkRequestFinished(int id, bool error)
{
	if(error)
	{
		if(subjectBuf != NULL)
		{
			delete	subjectBuf;
			subjectBuf = NULL;
		}
		if(network->error() == QHttp::Aborted)
		{
			targetBoardList.clear();
			qDebug() << "Network connection aborted!";
			isFavoritesUpdating = false;
		}
		else
		{
			QMessageBox::information(this, "Error", network->errorString());
		}
		if(isFavoritesUpdating && !targetBoardList.isEmpty())
		{
			targetBoardList.removeFirst();
			if(!targetBoardList.isEmpty())
			{
				QString	nextURL = targetBoardList.first()->first().URL;
				QUrl	nextSubject(nextURL.mid(0, nextURL.lastIndexOf('/', nextURL.lastIndexOf('/') - 1) + 1) + "subject.txt");
				subjectBuf = new QBuffer;
				subjectBuf->open(QIODevice::ReadWrite);
				QHttpRequestHeader	header("GET", nextSubject.path());
				header.setValue("Host", nextSubject.host());
				header.addValue("User-Agent", QString("Monazilla/1.00 (%1)").arg(APPNAME));
				network->setHost(nextSubject.host());
				favoriteUpdateID = network->request(header, "", subjectBuf);
				return;
			}
		}
		reloadSubjectButton->setDown(false);
		return;
	}
	// Loading board list
	if(id == boardListGetID)
	{
		boardBuf->close();
		QTextCodec *codec = QTextCodec::codecForName("Shift-JIS");
		char delimiter = '\n';
		boardModel->clear();
		QStringList colLabel;
		colLabel << tr("Board name") << tr("URL");
		boardModel->setHorizontalHeaderLabels(colLabel);
		// Add favorites to board list
		boardModel->invisibleRootItem()->setChild(0, 0, new QStandardItem(tr("Favorites")));
		boardModel->invisibleRootItem()->setChild(0, 1, new QStandardItem("Favorites"));
		
		QString data = codec->toUnicode(boardBuf->data());
		data.remove(0, data.indexOf("<B>"));
		data = data.remove("<BR>", Qt::CaseInsensitive);
		QList<QString> lines = data.split(delimiter);
		QList<QString>::iterator iter = lines.begin();
		QString line, link, name;
		int c_row = 1, b_row;
		while(iter != lines.end())
		{
			line = *iter;
			iter++;
			while(line.startsWith("<B>"))
			{
				line = line.left(line.indexOf("</B>"));
				QStandardItem *category = new QStandardItem(line.mid(3));
				line = *(iter++);
				b_row = 0;
				while(line.startsWith("<A HREF="))
				{
					QList<QStandardItem*> colList;
					int pos = line.indexOf("http://");
					QString	boardUrl = line.mid(pos, line.indexOf(">") - pos);
					if(!boardUrl.endsWith("TARGET=_blank") && !boardUrl.endsWith(".com/")
						&& !boardUrl.endsWith(".jp/") && !boardUrl.endsWith(".net/"))
					{
						QStandardItem *link = new QStandardItem(boardUrl);
						colList.append(link);
						pos = line.indexOf(">") + 1;
						QStandardItem *name = new QStandardItem(line.mid(pos, line.indexOf("</A>") - pos));
						link->setEditable(false);
						name->setEditable(false);
						category->setChild(b_row, 0, name);
						category->setChild(b_row, 1, link);
						b_row++;
					}
					line = *(iter++);
				}
				if(category->hasChildren())
				{
					boardModel->invisibleRootItem()->setChild(c_row, category);
					c_row++;
				}
			}
		}	
		boardList->setModel(boardModel);
		boardList->hideColumn(1);
		delete boardBuf;
	}
	//	Load thread list
	else if(id == subjectGetID)
	{
//		threadModelMutex.lock();
		
		statusBar()->clearMessage();
		QTextCodec *codec = QTextCodec::codecForName("Shift-JIS");
		threadModel->clear();
		QStringList	headers;
		headers << tr("URL") << tr("Title") << tr("Count") << tr("New");
		threadModel->setHorizontalHeaderLabels(headers);
		QString dat = codec->toUnicode(subjectBuf->data());
		delete	subjectBuf;
		subjectBuf = NULL;
//		subjectBufMutex.unlock();
		QList<QString> datlines = dat.split('\n');
		QList<QString>::iterator iter = datlines.begin();
		
		QString line;
		int row = 0;
		while((line = *(iter++)) != NULL)
		{
			QStandardItem *dat = new QStandardItem(currentBoard + "dat/" + line.mid(0, line.indexOf("<>")));
			int pos = line.indexOf("<>") + 2;
			QString	subjectText = line.mid(pos, line.lastIndexOf("(") - pos).replace("&amp;", "&");
			QStandardItem *subject = new QStandardItem(subjectText);
			pos = line.lastIndexOf("(") + 1;
			QStandardItem *count = new ResCountItem(line.mid(pos, line.length() - pos - 1));
			count->setTextAlignment(Qt::AlignRight);
			dat->setEditable(false);
			subject->setEditable(false);
			count->setEditable(false);
			threadModel->setItem(row, urlCol, dat);
			threadModel->setItem(row, subjectCol, subject);
			threadModel->setItem(row, countCol, count);
			threadModel->setItem(row, newCol, new	ResCountItem("0"));
			row++;
		}
		threadList->setModel(threadModel);
		restoreColumnWidth();

		threadModelMutex.unlock();
		threadList->setColumnHidden(urlCol, true);
		threadList->resizeRowsToContents();
		threadList->scrollToTop();
		reloadSubjectButton->setDown(false);
		reloadSubjectButton->setEnabled(true);
	}
	
	// Favorites subject
	else if((id == favoriteUpdateID) && isFavoritesUpdating)
	{
		QTextCodec	*codec = QTextCodec::codecForName("Shift-JIS");
		QStringList	datList = codec->toUnicode(subjectBuf->data()).split('\n');
		delete	subjectBuf;
		subjectBuf = NULL;
		// Build favorites model
		char	buf[1024 * 4];
		QList<Favorite>	*targetList = targetBoardList.takeFirst();
		QList<QStandardItem*>	items;
		QList<Favorite>::iterator	targetIter = targetList->begin();
		while(targetIter != targetList->end())
		{
			items.clear();
			QString	targetUrl = (*targetIter).URL;
			QStandardItem	*urlCol = new	QStandardItem(targetUrl);
			urlCol->setEditable(false);
			items.append(urlCol);
			QStandardItem	*nameCol = new	QStandardItem((*targetIter).name);
			nameCol->setEditable(false);
			items.append(nameCol);
			QString	url = (*targetIter).URL;
			// Count res number in cache file
			if((*targetIter).resCount == 0)
			{
				int	resCount = 0;
				if(url.contains('/'))
				{
					QStringList	urlItem = url.split("/");
					QFile	cacheFile(QDesktopServices::storageLocation(QDesktopServices::DataLocation)
						+ "/" + urlItem[2] + "/" + urlItem[urlItem.count() - 3] + "/" + urlItem[urlItem.count() - 1]);
					if(cacheFile.exists() && cacheFile.open(QIODevice::ReadOnly))
					{
						while(cacheFile.readLine(buf, sizeof(buf)) != -1)
						{
							resCount++;
						}
					}
				}
				(*targetIter).resCount = resCount;
			}
			QString	datNum = targetUrl.mid(targetUrl.lastIndexOf('/') + 1);
			int	line;
			if((line = datList.indexOf(QRegExp(QString("^%1.*").arg(datNum)))) >= 0)
			{
				QString	resNum = datList.at(line).mid(datList.at(line).lastIndexOf('(') + 1);
				resNum.chop(1);
				QStandardItem*	resCountItem = new ResCountItem(resNum);
				resCountItem->setEditable(false);
				items.append(resCountItem);
				int	unread = resNum.toInt() - (*targetIter).resCount;
				QStandardItem	*resNumItem = new ResCountItem(QString::number(unread));
				resNumItem->setEditable(false);
				items.append(resNumItem);
				// Count up unreads
				if(unread > 0)
				{
					unreadThreadNum++;
					unreadResNum += unread;
				}
			}
			else
			{
				items.append(new QStandardItem("Not found"));
				items.append(new ResCountItem(QString::number(-1)));
			}
			threadModel->appendRow(items);
			targetIter++;
		}
		if(targetBoardList.empty())
		{
			QStringList	labels;
			labels << tr("URL") << tr("Title") << tr("Count") << tr("New");
			threadModel->setHorizontalHeaderLabels(labels);
			threadModelMutex.unlock();
			threadList->setModel(threadModel);
			restoreColumnWidth();
#if 0
			int	col = 0;
			QList<int>::iterator	colIter = prefs->columnWidth.begin();
			while(colIter != prefs->columnWidth.end())
			{
				threadList->setColumnWidth(col, *colIter);
				colIter++,col++;
			}
#endif
			threadList->setColumnHidden(urlCol, true);
			threadList->resizeRowsToContents();
			threadList->sortByColumn(newCol);
			threadList->scrollToTop();
			reloadSubjectButton->setDown(false);
			reloadSubjectButton->setEnabled(true);
			statusBar()->showMessage(QString(tr("Unread thread : %1 Unread res : %2")).arg(QString::number(unreadThreadNum), QString::number(unreadResNum)));
			isFavoritesUpdating = false;
			if(isAutoUpdating)
			{
				updateTimer.start();
			}
			return;
		}
		else
		{
//			threadModelMutex.unlock();
			
			QString	nextURL = targetBoardList.first()->first().URL;
			QUrl	nextSubject(nextURL.mid(0, nextURL.lastIndexOf('/', nextURL.lastIndexOf('/') - 1) + 1) + "subject.txt");
			subjectBuf = new QBuffer;
			subjectBuf->open(QIODevice::ReadWrite);
			QHttpRequestHeader	header("GET", nextSubject.path());
			header.setValue("Host", nextSubject.host());
			header.addValue("User-Agent", QString("Monazilla/1.00 (%1)").arg(APPNAME));
			network->setHost(nextSubject.host());
			favoriteUpdateID = network->request(header, "", subjectBuf);
			return;
		}
	}
}

/*
	HTTP response header
*/
void MainWindowImpl::httpResponse(const QHttpResponseHeader &header)
{
	QStringList	cookies = header.allValues("Set-Cookie");
	QStringList::iterator	iter = cookies.begin();
	QMap<QString, QString>	map;
	while(iter != cookies.end())
	{
		QString	cookieLine = *iter;
		cookieLine.left(cookieLine.length() - cookieLine.indexOf(';'));
		if(cookieLine.contains('='))
		{
			QStringList	values = cookieLine.split('=');
			map.insert(values[0], values[1]);
		}
		iter++;
	}
	switch(header.statusCode())
	{
		case 401:
		case 404:
		case 503:
			statusBar()->showMessage(header.reasonPhrase());
			QMessageBox::information(this, tr("Error"), header.reasonPhrase());
			break;
		case 302:
			statusBar()->showMessage(tr("DAT file not found."));
	}
}

/*
	POST response header
*/
void	MainWindowImpl::postResponseHeader(const QHttpResponseHeader	&header)
{
	QDir	cacheRootDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
	if(!cacheRootDir.exists(postServer))
		cacheRootDir.mkdir(postServer);
	cacheRootDir.cd(postServer);
	QFile	cookieFile(cacheRootDir.filePath("cookie.txt"));
	if(!cookieFile.open(QIODevice::ReadWrite))
	{
		QMessageBox::information(this, tr("Error"), tr("Cannot open a file for cookie"));
		return;
	}
	QMap<QString, QString>	cookie;
	QDataStream	cookieStream(&cookieFile);
	cookieStream >> cookie;
	cookieFile.resize(0);
	QStringList	cookies = header.allValues("Set-Cookie");
	QStringList::iterator	iter = cookies.begin();
	while(iter != cookies.end())
	{
		QString	cookieLine = *iter;
		cookieLine = cookieLine.left(cookieLine.indexOf(';'));
		if(cookieLine.contains('='))
		{
			QStringList	cookieItems = cookieLine.split('=');
			cookie.insert(cookieItems[0], cookieItems[1]);
		}
		iter++;
	}
	cookieStream << cookie;
	cookieFile.close();
}

/*
	POST response received
*/
void	MainWindowImpl::postRequestFinished(int	reqId, bool	error)
{
	if(error)
	{
		QMessageBox::information(this, tr("Message POST error"), tr("Error occered for message posting."));
		return;
	}
	if(reqId == postReqId)
	{
		QTextCodec	*codec = QTextCodec::codecForName("Shift-JIS");
		QString	response = codec->toUnicode(postResponse->data());
		int	walker = response.indexOf("<title>", 0, Qt::CaseInsensitive) + 7;
		QString	reason = response.mid(walker, response.indexOf("</title>", 0, Qt::CaseInsensitive) - walker);
		walker = response.indexOf("</head>", walker, Qt::CaseInsensitive) + 4;
		QString	reasonText = response.mid(walker);
		QString	parsedReasonText = reasonText.replace("<br>", "\n");
		parsedReasonText.replace(QRegExp("<form.*/form>"), "");
		QRegExp	matchTag("<.*>");
		matchTag.setMinimal(true);
		parsedReasonText.replace(matchTag, "");
		delete	postResponse;
		
		// Cookie comfirmation
		if(response.contains("2ch_X:cookie") || response.contains(COOKIECONFIRM))
		{
			QFile	cookieFile(QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "/" + postServer + "/cache.txt");
			if(cookieFile.exists())
			{
				cookieFile.resize(0);
			}
			QMessageBox	cookieAlert(QMessageBox::Information, reason, parsedReasonText);
			QPushButton *okButton = cookieAlert.addButton(tr("I agree"), QMessageBox::AcceptRole);
			QPushButton	*cancelButton = cookieAlert.addButton(tr("Reject"), QMessageBox::RejectRole);
			cookieAlert.exec();
			// Accepting cookie confirmation
			if(cookieAlert.clickedButton() == okButton)
			{
				QString	target = resEditTab->currentWidget()->findChild<QLabel*>("editTitleLabel")->text();
				QUrl	postUrl(target);
				QStringList	tags = response.split("<");
				QMap<QString, QString>	params;
				QList<QString>::iterator	iter = tags.begin();
				QString	name, value;
				int	walker;
				while(iter != tags.end())
				{
					if((*iter).startsWith("input type=hidden") || (*iter).startsWith("input TYPE=hidden"))
					{
						walker = (*iter).indexOf("name=", 0, Qt::CaseInsensitive) + 5;
						name = (*iter).mid(walker, (*iter).indexOf(QRegExp("(\\s|>)"), walker) - walker).replace("\"", "");
						walker = (*iter).indexOf("value=", walker, Qt::CaseInsensitive) + 6;
						value = (*iter).mid(walker, (*iter).indexOf(QRegExp("(\\s|>)"), walker) - walker);
						qDebug() << "Name=" << name << " Value=" << value << "\n";
						if(!name.isEmpty())
						{
							if(value.startsWith("\""))
							{
								value.chop(1);
								postUrl.addEncodedQueryItem(name.toAscii(), codec->fromUnicode(value.mid(1)).toPercentEncoding());
							}
							else
							{
								postUrl.addQueryItem(name, value);
							}
						}
					}
					iter++;
				}
				postUrl.addEncodedQueryItem("submit", codec->fromUnicode(COOKIEACCEPTLABEL).toPercentEncoding());
				QHttpRequestHeader	header("POST", "/test/bbs.cgi?guid=ON");
				header.addValue("Host", postUrl.host());
				header.addValue("User-Agent", QString("Monazilla/1.00 (%1)").arg(APPNAME));
				header.addValue("Content-Length", QString(postUrl.encodedQuery().length()));
				if(!(target.contains('/')))
				{
					QMessageBox::warning(this, tr("POST Error"), tr("URL Invalid."));
					return;
				}
				QStringList	refItems = target.split("/");
				header.addValue("Referer", QString("http://%0/test/read.cgi/%1/%2/").arg(refItems[2], refItems[3], refItems[5]));
				// Add cookie to header
				QFile	cookieFile(QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "/" + postServer + "/cookie.txt");
				cookieFile.open(QIODevice::ReadOnly);
				QDataStream	cookieStream(&cookieFile);
				QMap<QString, QString>	cookie;
				cookieStream >> cookie;
				cookieFile.close();
				QStringList	cookieKeys = cookie.keys();
				QStringList::iterator	cookieIter = cookieKeys.begin();
				QString	cookieText;
				QTextStream	cookieTextStream(&cookieText);
				while(cookieIter != cookieKeys.end())
				{
					cookieTextStream << *cookieIter << "=" << cookie.value(*cookieIter) << "; ";
					cookieIter++;
				}
				header.addValue("Cookie", cookieText);
				qDebug() << "Post query=" << postUrl.encodedQuery() << "\n";
				postResponse = new QBuffer;
				postReqId = postConnection->request(header, postUrl.encodedQuery(), postResponse);
				return;
			}
			else if(cookieAlert.clickedButton() == cancelButton)
			{
			}
		}
		// POST succeed
		else if(response.contains(POSTCOMPLETE) || response.contains("2ch_X:true"))
		{
			TabWidgetImpl	*editTab = dynamic_cast<TabWidgetImpl*>(resEditTab->currentWidget());
			if(resEditTab->count() > 2)
			{
				delete	editTab;
				editTab = 0;
			}
			else
			{
				editTab->findChild<QPlainTextEdit*>("resEdit")->clear();
//				editTab->findChild<QLineEdit*>("nameForm")->clear();
				QLineEdit	*mailForm = editTab->findChild<QLineEdit*>("mailForm");
				if(prefs->defaultSage)
				{
					mailForm->setText("sage");
				}
				else
				{
					mailForm->setText(prefs->initialMail);
				}
			}
			reload();
		}
		// POST succeed with warning
		else if(response.contains("2ch_X:false"))
		{
			QMessageBox::information(this, reason, parsedReasonText);
			delete	resEditTab->currentWidget();
		}
		// POST server busy
		else if(response.contains(SERVERBUSY))
		{
			QMessageBox::information(this, reason, parsedReasonText);
		}
		// POST error
		else if(response.contains("2ch_X:error") || response.contains(SERVERERROR))
		{
			QMessageBox::warning(this, reason, parsedReasonText);
		}
	}
}
/******************************
	Res(message) editor pane
******************************/
/*
	Res(message) edit canceling
*/
void MainWindowImpl::reject()
{
	if(resEditTab->count() == 1 || resEditTab->currentIndex() == 0)
	{
		resEditTab->currentWidget()->findChild<QPlainTextEdit*>("resEdit")->clear();
		resEditTab->currentWidget()->findChild<QLineEdit*>("nameForm")->clear();
		resEditTab->currentWidget()->findChild<QLineEdit*>("mailForm")->setText(prefs->initialMail);
	}
	else
	{
		delete	resEditTab->currentWidget();
	}
}

/*
	Post message to server
*/
void MainWindowImpl::post()
{
	QPlainTextEdit *edit = resEditTab->currentWidget()->findChild<QPlainTextEdit*>("resEdit");
	QString message = edit->toPlainText();
	if(message.isEmpty())
	{
		QMessageBox::warning(this, tr("Post error"), tr("No messages for post"));
		return;
	}
	QString name = resEditTab->currentWidget()->findChild<QLineEdit*>("nameForm")->text();
	QString mail = resEditTab->currentWidget()->findChild<QLineEdit*>("mailForm")->text();
	QString	target = resEditTab->currentWidget()->findChild<QLabel*>("editTitleLabel")->text();
	QUrl	url(target);
	
	// Make query for posting
	QTextCodec *codec = QTextCodec::codecForName("Shift-JIS");
	QStringList keys = url.path().split('/');
	url.addQueryItem("bbs", keys[1]);
	keys[3].chop(4);
	url.addQueryItem("key", keys[3]);
	url.addQueryItem("time", "1");
	url.addEncodedQueryItem("submit", codec->fromUnicode(SUBMITLABEL).toPercentEncoding());
	url.addEncodedQueryItem("FROM", codec->fromUnicode(name).toPercentEncoding());
	url.addEncodedQueryItem("mail", codec->fromUnicode(mail).toPercentEncoding());
	url.addEncodedQueryItem("MESSAGE", codec->fromUnicode(message).toPercentEncoding());
	
	// Message posting
	postResponse = new QBuffer;
	postResponse->open(QIODevice::ReadWrite);
	postServer = url.host();
	postConnection->setHost(url.host());
	QHttpRequestHeader header("POST", "/test/bbs.cgi");
	header.addValue("Host", url.host());
	header.addValue("User-Agent", QString("Monazilla/1.00 (%1)").arg(APPNAME));
	header.addValue("Referer", target.left(target.indexOf("/dat")));
	QFile	cookieFile(QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "/" + postServer + "/cookie.txt");
	if(!(cookieFile.open(QIODevice::ReadWrite)))
	{
		QMessageBox::critical(this, tr("File error"), tr("Cannot open cookie file."));
		return;
	}
	QDataStream	cookieStream(&cookieFile);
	QMap<QString, QString>	cookie;
	cookieStream >> cookie;
	cookieFile.close();
	QStringList	cookieKeys = cookie.keys();
	QStringList::iterator	iter = cookieKeys.begin();
	QString	cookieText;
	QTextStream	cookieTextStream(&cookieText);
	while(iter != cookieKeys.end())
	{
		cookieTextStream << *iter << "=" << cookie.value(*iter) << "; ";
		iter++;
	}
	header.addValue("Cookie", cookieText);
	postReqId = postConnection->request(header, url.encodedQuery(), postResponse);
}

/*
 * Updating favorite list.
 */

void		MainWindowImpl::updateFavoriteList()
{
	QStandardItem	*favoritesItem = boardModel->item(0);
	if(favoritesItem != NULL)
	{
		boardList->setCurrentIndex(boardModel->indexFromItem(favoritesItem));
		boardSelected(boardModel->indexFromItem(favoritesItem));
	}
}
