#include "browsewidgetimpl.h"

extern	MainWindowImpl	*gMainWindow;
extern	Kimo2chPrefs	applicationPrefs;
/*
	Constructor
*/
BrowseWidgetImpl::BrowseWidgetImpl(QString	thread, QString	fontFamily, int	fontPointSize, QWidget	*parent, Qt::WFlags	f)
	:QWidget(parent, f)
{
	setupUi(this);
	
	resBrowser->setFontFamily(fontFamily);
	resBrowser->setFontPointSize(fontPointSize);
	isDatFalled = false;
	favorite = false;
	parentTabWidget = dynamic_cast<QTabWidget*>(parent);
	
	// Initialize network
	threadUrl = thread;
	QUrl	url(threadUrl);
	buffer = NULL;
	nextThreadBuffer = NULL;
	networkProxy = gMainWindow->getProxy();

	// Find and create cache file
	if(!threadUrl.startsWith("http://"))
	{
		QMessageBox::information(this, tr("Error"), tr("URL is invalid."));
		close();
		return;
	}
	QString	pathStr = threadUrl.mid(7);
	QStringList	pathItem = pathStr.split("/");
	QDir	cacheRootDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
	cacheRootDir.cdUp();
	if(!cacheRootDir.exists(APPNAME))
	{
		cacheRootDir.mkdir(APPNAME);
	}
	cacheRootDir.cd(APPNAME);
	if(!cacheRootDir.exists(pathItem[0]))
	{
		cacheRootDir.mkdir(pathItem[0]);
	}
	cacheRootDir.cd(pathItem[0]);
	if(!cacheRootDir.exists(pathItem[1]))
	{
		cacheRootDir.mkdir(pathItem[1]);
	}
	cacheRootDir.cd(pathItem[1]);
	datFile = new QFile(cacheRootDir.absoluteFilePath(pathItem[3]));
	if(!datFile->open(QIODevice::ReadWrite))
	{
		qDebug() << "File open error at constructor.";
	}
//	cache = new	QByteArray;
	cache.resize(CACHESIZE);
	datFile->seek(0);
	cache = datFile->read(CACHESIZE);
	
	// Readed res count
	QTextCodec	*codec = QTextCodec::codecForName("Shift-JIS");
	QString	datString = codec->toUnicode(cache);
	readedCount = datString.count('\n');
	// Initialize variables
	lastModified = "";
	// Signal / Slot connection
	connect(&connection, SIGNAL(requestFinished(const int, const bool)), this, SLOT(httpRequestFinished(const int, const bool)));
	connect(&connection, SIGNAL(responseHeaderReceived(const QHttpResponseHeader&)), this, SLOT(httpResponseHeader(const QHttpResponseHeader&)));
	connect(&nextThreadConnection, SIGNAL(requestFinished(const int, const bool)), this, SLOT(findNextThreads(const int, const bool)));
//	viewUpdate();
}

/*
	Destructor
*/
BrowseWidgetImpl::~BrowseWidgetImpl()
{
#if 0
	if(connection != NULL)
	{
		connection->abort();
		delete	connection;
		connection = NULL;
	}
	if(buffer != NULL)
	{
		delete	buffer;
		buffer = NULL;
	}
	if(nextThreadConnection != NULL)
	{
		nextThreadConnection->abort();
		delete	nextThreadConnection;
		nextThreadConnection = NULL;
	}
	if(nextThreadBuffer != NULL)
	{
		delete	nextThreadBuffer;
		nextThreadBuffer = NULL;
	}
#endif
	connection.disconnect(this);
	connection.abort();
	nextThreadConnection.disconnect(this);
	nextThreadConnection.abort();
	datFile->resize(0);
	datFile->write(cache);
	datFile->close();
	delete	datFile;
}

/*
	Reload browser
*/
void	BrowseWidgetImpl::reload()
{
	int	state = connection.state();
	if(state == QHttp::Sending || state == QHttp::Reading || state == QHttp::Closing)
	{
		oldBuffer = buffer;
		connection.abort();
		return;
	}
	
	if(readedCount >= 1001)
	{
		viewUpdate();
		if(gMainWindow->isAutoReloading())
		{
			downloadSubjectForNextThread();
		}
		return;
	}
	
	parentTabWidget->setTabText(parentTabWidget->indexOf(this), tr("Loading..."));
	buffer = new QBuffer;
	if(!buffer->open(QIODevice::ReadWrite))
	{
		qDebug() << "Buffer open error at reload";
		return;
	}
	QUrl	url(threadUrl);
	
	connection.setProxy(*networkProxy);
	connection.setHost(url.host());
	QHttpRequestHeader	header("GET", url.path());
	header.setValue("Host", url.host());
	header.addValue("User-Agent", userAgent);
	if(cache.size() > 0)
	{
		header.addValue("If-Modified-Since", lastModified);
		header.addValue("Range", QString("bytes=%1-").arg(cache.size() - 1));
	}
	datGetId = connection.request(header, 0, buffer);
	qDebug() << "reload URL =" << threadUrl;
}

/*
	Browser view update(redraw)
*/
void	BrowseWidgetImpl::viewUpdate()
{
	if(cache.size() == 0)
	{
		return;
	}
#ifdef QT_CP932_SUPPORT

	QTextCodec	*codec = QTextCodec::codecForName("Shift-JIS");
	datFile->seek(0);
	QString	plainDat = codec->toUnicode(cache);

#else

	QTextCodec	*codec = CP932Codec::codecForMib(2024);
	datFile->seek(0);
	QString	plainDat = codec->toUnicode(cache);

#endif

#if 0
	/*
	 * Convert Shift_JIS(CP932) to UTF-8 using iconv
	 */
	char	convertedText[ICONVBUFSIZE];	// Max dat file size is 512KB
	char	*bufText = convertedText;
	size_t	bufSize = ICONVBUFSIZE;
	if(cache.size() == 0)
	{
		return;
	}
	QByteArray datData = cache; //datFile->readAll();
	datData.append('\0');
	char	*sourceText = datData.data();
	size_t	srcSize = strlen(sourceText);
	iconv_t	textConverter;
	if((textConverter = iconv_open("UTF-8//IGNORE", "CP932")) == (iconv_t)(-1))
	{
		qDebug() << "Text converter is NULL";
	}
	errno = 0;
	iconv(textConverter, &sourceText, &srcSize, &bufText, &bufSize);
	int	err = errno;
	iconv_close(textConverter);
	switch(err)
	{
		case E2BIG:
			QMessageBox::warning(this, tr("Text convert"), tr("Text convert is out of buffer."));
			break;
		
		case EILSEQ:
			QMessageBox::warning(this, tr("Text convert"), tr("Illegal sequence arrived."));
			break;
		
		case EINVAL:
			QMessageBox::warning(this, tr("Text convert"), tr("Invalid string arrived."));
			break;
	}
	*bufText = '\0';
	QString	plainDat = QString::fromUtf8(convertedText);

#endif	// QT_CP932_SUPPORT

	plainDat.chop(1);
	if(plainDat.contains('\n'))
	{
		dat = plainDat.split('\n');
	}
	else
	{
		dat.append(plainDat);
	}
	resBrowser->setDat(dat, readedCount);
	QString	firstRes = plainDat.left(plainDat.indexOf('\n'));
	if(!firstRes.contains("<>"))
	{
		QMessageBox::warning(this, tr("File error"), tr("DAT file broken. First res invalid."));
		return;
	}
	QStringList	resItems = firstRes.split("<>");
	if(resItems.count() < 5)
	{
		QMessageBox::warning(this, tr("File error"), tr("DAT file broken. First res not contains title."));
		return;
	}
	threadTitle = resItems[4].replace("&amp;", "&&");
	readedCount = plainDat.count('\n') + 1;
	QList<Favorite>	*favoriteList = gMainWindow->getFavoriteList();
	if(favorite && (favoriteList != NULL))
	{
		QList<Favorite>::iterator	iter = favoriteList->begin();
		while(iter != favoriteList->end())
		{
			if((*iter).URL == threadUrl)
			{
				(*iter).resCount = readedCount;
				return;
			}
			iter++;
		}
	}
	if(readedCount >= 1001)
	{
		isDatFalled = true;
		datFile->write(cache);
		datFile->flush();
	}
}

void	BrowseWidgetImpl::closeTab()
{
	QTabWidget	*parentTab = dynamic_cast<QTabWidget*>(this->parent());
	parentTab->removeTab(parentTab->currentIndex());
}

/*
	Response header analyze
*/
void	BrowseWidgetImpl::httpResponseHeader(const QHttpResponseHeader	header)
{
	lastStatus = header.statusCode();
	qDebug() << lastStatus;
	needRefresh = false;
	switch(lastStatus)
	{
		case	200:
			datFile->resize(0);
			cache.resize(0);
			readedCount = 0;
		case	206:
			lastModified = header.value("Last-Modified");
			needRefresh = true;
			isDatFalled = false;
			break;
		
		case	302:
			qDebug() << "dat file download failed." << endl;
			if(header.value("Location").contains("403/"))
			{
				QMessageBox::information(this, tr("Access denied"),
					QString(tr("BBS server access denied.\nInformation page is %1")).arg(header.value("Location")));
			}
			isDatFalled = true;
			needRefresh = false;
			reloadCompleted();
			break;
		
		case	304:
			needRefresh = false;
			reloadCompleted();
			break;
		
		case	404:
			qDebug() << "dat file not found." << endl;
			isDatFalled = true;
			needRefresh = false;
			reloadCompleted();
			break;
		
		default:
			qDebug() << "Result code =" << lastStatus;
			needRefresh = false;
			isDatFalled = false;
			reloadCompleted();
			break;
	}
}

void	BrowseWidgetImpl::httpRequestFinished(const int	id, const bool	error)
{
	int	errorValue = connection.error();
	if(error || isDatFalled)
	{
		viewUpdate();
		if(errorValue == QHttp::Aborted)
		{
			oldBuffer->close();
			delete	oldBuffer;
			oldBuffer = NULL;
		}
		reloadCompleted();
		return;
	}
	if(id == datGetId && buffer)
	{
		buffer->seek(0);
		if(buffer->bytesAvailable() > 0)
		{
			// Abone check
			if(cache.size() > 0 && !buffer->data().startsWith('\n'))
			{
				qDebug() << "Abone occered";
				cache.resize(0);
				datFile->resize(0);
				buffer->close();
				delete	buffer;
				buffer = NULL;
				reload();
				return;
			}
			else
			{
				cache.append(buffer->data().startsWith("\n") ? buffer->data().mid(1) : buffer->data());
				if(datFile->size() == 0 || datModifiedTime.addSecs(10 * 60) <= QDateTime::currentDateTime())
				{
					datFile->resize(0);
					datFile->write(cache);
					datFile->flush();
					datModifiedTime = QFileInfo(*datFile).lastModified();
				}
				buffer->close();
				delete	buffer;
				buffer = NULL;
				qDebug() << "Cache size =" << cache.size();
			}
		}
		if(needRefresh)
		{
			viewUpdate();
			qDebug() << "View updated";
		}
	}
	parentTabWidget->setTabText(parentTabWidget->indexOf(this), threadTitle);
	parentTabWidget->setTabToolTip(parentTabWidget->indexOf(this), threadTitle);
	QTextEdit	*searchEdit = gMainWindow->findChild<QTextEdit*>("searchEdit");
	if(searchEdit)
	{
		searchEdit->setText(threadTitle);
	}
	reloadCompleted();
	qDebug() << "Reload completed";
}

void	BrowseWidgetImpl::downloadSubjectForNextThread()
{
	if(nextThreadConnection.state() == QHttp::Reading)
	{
		reloadCompleted();
		return;
	}
	nextThreadConnection.abort();
	if(nextThreadBuffer)
	{
		delete	nextThreadBuffer;
		nextThreadBuffer = NULL;
	}
	QUrl	subjectUrl(threadUrl.mid(0, threadUrl.lastIndexOf("dat/")) + "subject.txt");
//	nextThreadBuffer.size(0);
	nextThreadBuffer = new	QBuffer;
	if(!nextThreadBuffer->open(QIODevice::ReadWrite))
	{
		qDebug() << "Can't open buffer in downloadSubjectForNextThread()";
		return;
	}
	QHttpRequestHeader	header;
	nextThreadConnection.setProxy(*networkProxy);
	nextThreadConnection.setHost(subjectUrl.host());
	header.setRequest("GET", subjectUrl.path());
	header.addValue("Host", subjectUrl.host());
	header.addValue("User-Agent", QString("Monazilla/1.00(%1)").arg(APPNAME));
	subjectGetId = nextThreadConnection.request(header, 0, nextThreadBuffer);
}

void	BrowseWidgetImpl::findNextThreads(const int	id, const bool	error)
{
	if(error)
	{
		reloadCompleted();
		return;
	}
	if(id == subjectGetId && nextThreadBuffer)
	{
		//QTextCodec	*codec = QTextCodec::codecForName("Shift-JIS");
		QTextCodec	*codec = CP932Codec::codecForName("Windows-31J");
		QString	subjectTxt = codec->toUnicode(nextThreadBuffer->data());
		nextThreadBuffer->close();
		delete	nextThreadBuffer;
		nextThreadBuffer = NULL;
		if(subjectTxt.at(subjectTxt.size() - 1) == '\n')
			subjectTxt.chop(1);
		QStringList	subject = subjectTxt.split('\n');
		
		QMap<QString, QString>	foundThreads;
		QString	searchText = threadTitle;
		// Find SAME NAME THREAD
		for(QStringList::const_iterator	iter = subject.constBegin(); iter != subject.constEnd(); iter++)
		{
			if(!((*iter).contains("<>")))
			{
				QMessageBox::information(this, tr("Error"), tr("subject.txt is broken."));
				return;
			}
			QStringList	subjectItem = (*iter).split("<>");
			if(subjectItem[1].right(6) != "(1001)" && subjectItem[1].contains(searchText))
			{ 
				foundThreads.insert(subjectItem[0], subjectItem[1]);
			}
		}
		// Find same name thread EXCLUDE text IN BRACKETS.
		if(foundThreads.isEmpty())
		{
			searchText = searchText.mid(searchText.indexOf(QRegExp("(\\)\\]】)")));
			searchText = searchText.mid(0, searchText.lastIndexOf(QRegExp("(\(\[【)")));
			for(QStringList::const_iterator	iter = subject.constBegin(); iter != subject.constEnd(); iter++)
			{
				if(!(*iter).contains("<>"))
				{
					QMessageBox::warning(this, tr("Error"), tr("subject.txt is broken."));
					return;
				}
				QStringList	subjectItem = (*iter).split("<>");
				if(subjectItem[1].right(6) != "(1001)" && subjectItem[1].contains(searchText))
				{
					foundThreads.insert(subjectItem[0], subjectItem[1]);
				}
			}
		}
		// Find like name thread (match forward)
		if(foundThreads.isEmpty())
		{
			searchText = searchText.left(searchText.length() / 2);
			for(QStringList::const_iterator	iter = subject.constBegin(); iter != subject.constEnd(); iter++)
			{
				if(!(*iter).contains("<>"))
				{
					QMessageBox::warning(this, tr("Error"), tr("subject.txt is broken."));
					return;
				}
				QStringList	subjectItem = (*iter).split("<>");
				if(subjectItem[1].right(6) != "(1001)" && subjectItem[1].contains(searchText))
				{
					foundThreads.insert(subjectItem[0], subjectItem[1]);
				}
			}
		}
		// for auto reloading
		if(gMainWindow->isAutoReloading())
		{
			if(foundThreads.isEmpty())
			{
				reloadCompleted();
				return;
			}
			else
			{
				QMap<QString, QString>::const_iterator	iter = foundThreads.constBegin();
				gMainWindow->openLink(QUrl(threadUrl.left(threadUrl.lastIndexOf(QRegExp("/")) + 1) + iter.key()));
				reloadCompleted();
				return;
			}
		}
		// If found no next thread
		if(foundThreads.isEmpty())
		{
			reloadCompleted();
			return;
		}
		// Show dialog for select a next thread
		Ui::NextThreadDialog	ui;
		QDialog	*nextThreadDialog = new	QDialog(this);
		ui.setupUi(nextThreadDialog);
		QStandardItemModel	nextThreadModel;
		QMap<QString, QString>::const_iterator	threadIter = foundThreads.constBegin();
		while(threadIter != foundThreads.constEnd())
		{
			QList<QStandardItem*>	threadItemList;
			threadItemList << new QStandardItem(threadIter.value()) << new QStandardItem(threadIter.key());
			nextThreadModel.appendRow(threadItemList);
			threadIter++;
		}
		QListView	*nextThreadListView = nextThreadDialog->findChild<QListView*>("nextThreadListView");
		nextThreadListView->setModel(&nextThreadModel);
		int	result = nextThreadDialog->exec();
		if(result == QDialog::Rejected)
		{
			reloadCompleted();
			return;
		}
		QString	selectedDatNum = nextThreadListView->currentIndex().sibling(nextThreadListView->currentIndex().row(), 1).data().toString();
		gMainWindow->openLink(threadUrl.left(threadUrl.lastIndexOf('/') + 1) + selectedDatNum);
		delete	nextThreadDialog;
	}
	reloadCompleted();
}

void	BrowseWidgetImpl::clearCache()
{
	cache.resize(0);
	datFile->resize(0);
}