#ifndef __BROWSEWIDGETIMPL_H__
#define __BROWSEWIDGETIMPL_H__

#include <QtGui>
#include <QtNetwork>
#include <iconv.h>
#include <errno.h>
#include <time.h>
#include "globals.h"
#include "ui_nextthreaddialog.h"
#include "ui_browsewidget.h"
#include "src/kimo2chtextbrowser.h"
#include "src/mainwindowimpl.h"
#include "src/xmlprefshandler.h"
#include "src/cp932codec.h"

class	BrowseWidgetImpl : public	QWidget, public	Ui::BrowseWidget
{
Q_OBJECT;
public:
	BrowseWidgetImpl(QString	thread, QString	fontFamily, int	fontPointSize, QWidget	*parent = 0, Qt::WFlags	f = 0);
	virtual	~BrowseWidgetImpl();
	QString	getThreadUrl() { return	threadUrl; };
	
	bool	isFavorite()	{ return	favorite; }
	void	setFavorite(bool	state)	{ favorite = state; }
//	void	setAutoReload(bool	state)	{ autoReload = state; }
	void	downloadSubjectForNextThread();
	void	setProxy(QNetworkProxy	*proxy)	{ networkProxy = proxy; }
	
	QString	getTitle() { return	threadTitle; };
	QString	getUrl() { return	threadUrl; };
	
	
public slots:
	void	reload();		// Reload thread
	void	clearCache();	// Clear cache

private slots:
	void	viewUpdate();	// Browser view update
	void	closeTab();		// Close tab contains this widget
	void	httpResponseHeader(const QHttpResponseHeader	header);
	void	httpRequestFinished(const int	id, const bool	error);
	void	setUrl(const QString	targetUrl) { threadUrl = targetUrl; };
	void	findNextThreads(const int	id, const bool	error);
	
signals:
	void	reloadCompleted();
	void	openNextThread(QString	url);	// Open next thread
	void	requestLink(QUrl	url);
	
private:
	QTabWidget	*parentTabWidget;	// Parent widget for this tab
	
	QString	threadUrl;		// Target thread URL
	QString	threadTitle;	// Thread title
	int		readedCount;	// Readed res(message) count
	QByteArray	cache;		// Dat file cache
	
	QHttp	connection;	// HTTP connection instance
	QHttp	nextThreadConnection;	// HTTP connection for find next thread
	QNetworkProxy	*networkProxy;	// HTTP proxy
	QBuffer	*buffer;		// HTTP message body buffer
	QBuffer	*oldBuffer;		// HTTP buffer at HTTP aborted
	QBuffer	*nextThreadBuffer;	// HTTP buffer for find next thread
	int		lastStatus;		// HTTP response code
	int		datGetId;		// HTTP dat file get id
	int		subjectGetId;	// HTTP subject.txt get id
	QFile	*datFile;		// dat cache file
	QStringList	dat;
	QString	lastModified;	// Last modified date in last server response header
	QDateTime	datModifiedTime;	// DAT file last modified time
	bool	needRefresh;	// Need refresh flag
	bool	isDatFalled;	// Flag of dat falled
	bool	favorite;		// Thread in favorites flag
//	bool	autoReload;		// State of auto reload
};

#endif // __BROWSEWIDGETIMPL_H__
