#include "cp932codec.h"

static	CP932Codec	*gCodec;

CP932Codec::CP932Codec()
	: QTextCodec()
{
	if((fromcd = iconv_open("CP932//IGNORE", "UTF-8")) == (iconv_t)-1)
	{
		qDebug() << "iconv_open error";
	}
	if((tocd = iconv_open("UTF-8//IGNORE", "CP932")) == (iconv_t)-1)
	{
		qDebug() << "iconv_open error";
	}
}

CP932Codec::~CP932Codec()
{
	if(fromcd != 0)
	{
		iconv_close(fromcd);
		fromcd = 0;
	}
	if(tocd != 0)
	{
		iconv_close(tocd);
		tocd = 0;
	}
}

QByteArray	CP932Codec::convertFromUnicode(const QChar	*input, int	number, ConverterState*	/*state */)	const
{
	QByteArray	inputArray(QString(input, number).toUtf8());
	size_t	inputLength = inputArray.length();
	char	*inputChars = inputArray.data();
	char	outputStr[inputLength];
	char	*outputPtr = outputStr;
	size_t	inSize = inputLength;
	size_t	outSize = inputLength;
	int	err = 0;
	size_t	convCount = iconv(fromcd, &inputChars, &inSize, &outputPtr, &outSize);
	err = errno;
	switch(err)
	{
		case	EINVAL:
			qDebug() << "iconv error EINVAL";
			break;
		case	E2BIG:
			qDebug() << "iconv error E2BIG";
			break;
		case	EILSEQ:
			qDebug() << "iconv error EILSEQ";
			break;
	}
	QByteArray	convertedBytes(outputStr, inputLength - outSize);
	return	convertedBytes;
}

QString	CP932Codec::convertToUnicode(const char	*chars, int	len, ConverterState	* /*state*/) const
{
	char	inputStr[len + 1];
	char	*inputChars = inputStr;
	char	outputStr[len * 3];
	char	*outputChars = outputStr;
	size_t	inLength;
	inLength = len;
	size_t	outLength;
	outLength = len * 3;
	strncpy(inputStr, chars, len);
	int	err = 0;
	size_t	convCount = iconv(tocd, &inputChars, &inLength, &outputChars, &outLength);
	err = errno;
	switch(err)
	{
		case	E2BIG:
			qDebug() << "iconv error E2BIG";
			break;
		case	EILSEQ:
			qDebug() << "iconv error EILSEQ";
			break;
		case	EINVAL:
			qDebug() << "iconv error EINVAL";
			break;
	}
	return	QString::fromUtf8(outputStr, len *3 - outLength);
}

QTextCodec	*CP932Codec::codecForMib(int	mib)
{
	if(mib == 2024)
	{
		if(!gCodec)
		{
			gCodec = new	CP932Codec();
			qDebug() << "Codec create";
		}
		return	gCodec;
	}
	return	QTextCodec::codecForMib(mib);
}

QTextCodec	*CP932Codec::codecForName(const	char	*name)
{
	if(strncmp(name, "Windows-31J", 11))
	{
		if(!gCodec)
		{
			gCodec = new	CP932Codec();
			qDebug() << "Codec create";
		}
		return	gCodec;
	}
	return	QTextCodec::codecForName(name);
}
// place your code here
