#include "kimo2chtextbrowser.h"

extern	Kimo2chPrefs	applicationPrefs;

Kimo2chTextBrowser::Kimo2chTextBrowser(QWidget	*parent, Kimo2chTextBrowser	*parentPopupWidget)
	:QTextBrowser(parent)
	,parentPopup(parentPopupWidget)
{
	popup = NULL;
	popupBrowser = NULL;
	cm = NULL;
	setAttribute(Qt::WA_Hover, true);
	isEnterCursor = false;
}

Kimo2chTextBrowser::~Kimo2chTextBrowser()
{
#if 0
	if(parserThread)
	{
		if(parserThread->isRunning())
		{
			parserThread->exit();
			parserThread->wait();
		}
		delete	parserThread;
		parserThread = NULL;
	}
	if(popup)
	{
		Kimo2chTextBrowser	*browser = popup->findChild<Kimo2chTextBrowser*>("Popup");
		if(browser)
		{
			delete	browser;
			browser = NULL;
			delete	popup;
			popup = NULL;
		}
	}
#endif
}

void	Kimo2chTextBrowser::setDat(QStringList	dat, int	count)
{
#if 0
	readedCount = count;
	if(parserThread->isRunning())
	{
		parserThread->terminate();
	}
	datRef = dat;
	parserThread->start();
#endif
	readedCount = count;
	datRef = dat;
	idMap.clear();
	QString	parsedText;
	QTextStream	parsedStream(&parsedText);
	// Font family and size settings
	parsedStream << "<body style=\"font-family : " << browserFontFamily << "; font-size :" << browserFontPointSize << "pt;\">";
	
	QStringList::iterator	iter = datRef.begin();
	int	i = 1;
	while(iter != datRef.end())
	{
		QString	datLine = *iter;
		QStringList	resItems = datLine.split("<>");
		if(resItems.count() < 4)
		{
//			QMessageBox::warning(browser, tr("File error"), tr("DAT file is broken. Res format invalid."));
			return;
		}
		QString	idAnchor;
		int	idstart = resItems[2].indexOf("ID:");
		if(idstart >= 0)
		{
			int idlength = resItems[2].indexOf(QRegExp("\\s"), idstart) - idstart;
			QString	id = resItems[2].mid(idstart, (idlength < 0) ? -1 : idlength);
			idMap.insert(id, i);
		}
		parsedStream << "<a name=\"res" << i << "\">" << i << "</a> " << ResParser::parse(datLine);
		iter++;
		if(i > 1 && i == readedCount && iter != datRef.end())
		{
			parsedStream << "<hr>" << tr("New message(s)") << "<hr>";
		}
		i++;
	}
	parsedStream << "<br>";	// Add last blank line for margin.
	setHtml(parsedText);
	scrollToAnchor(QString("res%1").arg(readedCount));
}

void	Kimo2chTextBrowser::setParsedText()
{
	setHtml(parsedText);
	scrollToAnchor(QString("res%1").arg(readedCount));
}

/*
	Anchor rewrite
*/
QString	Kimo2chTextBrowser::rewriteAnchor(const QString	originalText)
{
	QString	tempText(originalText);
	tempText.replace("‾", "~");
	// Res number anchor rewrite
	tempText.remove(QRegExp("<a \\S+ target=\"_blank\">"));	// Remove start tags
	int	pos = 0;
	while((pos = tempText.indexOf(QRegExp("&gt;&gt;(-\\,\\d{,4})*</a>"), pos)) != -1)
	{
		int	endpos = tempText.indexOf("</a>", pos);
		QString	anchor = tempText.mid(pos + 8, endpos - pos - 8);
		tempText.insert(pos, QString("<a href=\"#res%1\">").arg(anchor));
		pos = endpos + anchor.length() + 15;
	}
	// Anchor insert to URLs
	pos = 0;
	while((pos = tempText.indexOf(QRegExp("(http|ttp|tp)://"), pos)) != -1)
	{
		int	endpos = tempText.indexOf(QRegExp("([^a-zA-Z0-9~%&#=\\(\\)\\:/\\-\\.\\?\\_])"), pos);
		tempText.insert(endpos, "</a>");
		QString	url = tempText.mid(pos, endpos - pos);
		url = url.mid(url.indexOf(":"));
		tempText.insert(pos, QString("<a href=\"http%1\">").arg(url));
		pos = endpos + url.length() + 15;
	}
	return	tempText;
}

/*
	Anchor res(message) popup
*/
void	Kimo2chTextBrowser::mouseMoveEvent(QMouseEvent	*e)
{
	QString	anchor = anchorAt(e->pos());
	if(!(anchor.startsWith("#")) || (popup != NULL && popup->isVisible()))
	{
		QTextBrowser::mouseMoveEvent(e);
	}
	else
	{
		QDesktopWidget	desktop;
		if(popup == NULL)
		{
			popup = new QWidget(this, Qt::ToolTip);
			popup->setAttribute(Qt::WA_DeleteOnClose, true);
			popupBrowser = new Kimo2chTextBrowser(popup, this);
			connect(popupBrowser, SIGNAL(linkClicked(QUrl)), this, SLOT(relayLink(QUrl)));
			popupBrowser->setObjectName("Popup");
			popupBrowser->setFont(font());
			popupBrowser->setFontFamily(fontFamily());
			popupBrowser->setFontPointSize(fontPointSize());
			popupBrowser->setLineWrapMode(QTextEdit::NoWrap);
			popupBrowser->setDatData(datRef, idMap);
			popup->hide();
		}
		QString	parsedText;
		QTextStream	parsedStream(&parsedText);
		// Hover cursor at res anchors
		if(anchor.startsWith("#res"))
		{
			int	startRes, endRes;
			QString	res = anchor.mid(4);
			if(res.contains('-'))
			{
				startRes = res.split('-').at(0).toInt();
				endRes = res.split('-').at(1).toInt();
			}
			else
			{
				startRes = endRes = res.toInt();
			}
			for(int	num = startRes; num <= endRes; num++)
			{
				if(num > datRef.count())
				{
					popup->hide();
					return;
				}
				QStringList	resItems = datRef.at(num - 1).split("<>");
				parsedStream << "<body style=\"font-family : " << applicationPrefs.browserFontFamily
					<< "; font-size : " << applicationPrefs.browserFontSize << "pt; \">";
				parsedStream << num << ResParser::parse(datRef.at(num - 1));
			}
		}
		else if(anchor.startsWith("#ID"))
		{
			QString	id = anchor.mid(1);
			if(id.length() < 11)
			{
				QToolTip::hideText();
				return;
			}
			parsedStream << " <body style=\"font-family : " << applicationPrefs.browserFontFamily
				<< "; font-size : " << applicationPrefs.browserFontSize << "pt; \">";
			QList<int>	resList = idMap.values(id);
			while(!resList.isEmpty())
			{
				int	resNum = resList.takeLast();
				QStringList	resItems = datRef.at(resNum - 1).split("<>");
				parsedStream << resNum << ResParser::parse(datRef.at(resNum - 1));
			}
		}
		else
		{
			return;
		}
		isEnterCursor = false;
		popupBrowser->setText(parsedText);
		QSizeF	size = popupBrowser->document()->size();
		int	margin = verticalScrollBar()->width() + 8;
		popupBrowser->setFixedWidth(size.width() + margin < (desktop.width() / 2)? size.width() + margin : desktop.width() / 2);
		popupBrowser->setFixedHeight(size.height() + margin < desktop.height() ? size.height() + margin : desktop.height());
		QPoint	pos = e->globalPos();
		popup->move(pos.x() + size.width() < desktop.width() ? pos.x() : (pos.x() - size.width() > 0 ? pos.x() - size.width() : 0)
			, pos.y() + size.height() < desktop.height() ? pos.y() : desktop.height() - popupBrowser->height());
		popupBrowser->isEnterCursor = false;
		popup->show();
	}
	QTextBrowser::mouseMoveEvent(e);
}

/*
	Link clicked
*/
void	Kimo2chTextBrowser::mouseReleaseEvent(QMouseEvent	*e)
{
	// 
	QString	anchor = anchorAt(e->pos());
	if(anchor == "")
	{
		QTextBrowser::mouseReleaseEvent(e);
	}
	else
	{
		if(anchor.startsWith("#"))
		{
			if(anchor.startsWith("#res"))
			{
				this->scrollToAnchor(anchor.mid(1));
			}
		}
		else
		{
			QUrl url = QUrl::fromEncoded(anchor.toAscii());
			linkClicked(url);
		}
	}
}

/*
	Context menu
*/
void	Kimo2chTextBrowser::contextMenuEvent(QContextMenuEvent	*e)
{
//	QTextBrowser::contextMenuEvent(e);
	cm = createStandardContextMenu(e->globalPos());
	copyURLAction = cm->addAction(QString(tr("Copy this link")), this, SLOT(copyAnchor()));
	QString	anchor = anchorAt(e->pos());
	if(anchor.startsWith("http"))
	{
		anchorText = anchor;
	}
	else
	{
		copyURLAction->setEnabled(false);
	}
	if(anchor.startsWith("#ID:"))
	{
		copyAction = cm->addAction(tr("Copy this ID"), this, SLOT(copyHeadItem()));
		copyAction->setData(anchor.mid(4));
		
		aboneAction = cm->addAction(QString(tr("Abone %1").arg(anchor.mid(4))), this, SLOT(addAboneId()));
		aboneAction->setData(anchor);
	}
	else if(anchor.startsWith("#NAME:"))
	{
		copyAction = cm->addAction(QString(tr("Copy this name %1").arg(anchor.mid(6))), this, SLOT(copyHeadItem()));
		copyAction->setData(anchor.mid(6));
		
		aboneAction = cm->addAction(QString(tr("Abone name for %1").arg(anchor.mid(6))), this, SLOT(addAboneName()));
		aboneAction->setData(anchor.mid(6));
	}
	else if(anchor.startsWith("#MAIL:"))
	{
		copyAction = cm->addAction(tr("Copy this mail"), this, SLOT(copyHeadItem()));
		copyAction->setData(anchor.mid(6));
		
		aboneAction = cm->addAction(QString(tr("Abone mail for %1").arg(anchor.mid(6))), this, SLOT(addAboneMail()));
		aboneAction->setData(anchor.mid(6));
	}
	cm->exec(e->globalPos());
	delete	cm;
	cm = NULL;
}

/*******************************
	Slots
*******************************/

/*
 * relayLink
 */

void	Kimo2chTextBrowser::relayLink(QUrl	url)
{
	linkClicked(url);
}

/*
 * copyAnchor
 */
void	Kimo2chTextBrowser::copyAnchor()
{
	qDebug() << "Copy URL";
	QClipboard	*cb = QApplication::clipboard();
	cb->setText(anchorText);
	delete	copyURLAction;
}

// Copy
void	Kimo2chTextBrowser::copyHeadItem()
{
	QClipboard	*cb = QApplication::clipboard();
	cb->setText(copyAction->data().toString());
	delete	copyAction;
	delete	aboneAction;
	delete	copyURLAction;
}


/*
 * Add to Abone lists
 */

void	Kimo2chTextBrowser::addAboneId()
{
	ResParser::aboneId.insert(aboneAction->data().toString().mid(1), true);
	delete	aboneAction;
	delete	copyAction;
	delete	copyURLAction;
//	ResParser::aboneId.insert(aboneIdAction->data().toString().mid(1), true);
}

void	Kimo2chTextBrowser::addAboneName()
{
	ResParser::aboneName.insert(aboneAction->data().toString(), true);
	delete	aboneAction;
	delete	copyAction;
	delete	copyURLAction;
}

void	Kimo2chTextBrowser::addAboneMail()
{
	ResParser::aboneMail.insert(aboneAction->data().toString(), true);
	delete	aboneAction;
	delete	copyAction;
	delete	copyURLAction;
}

void	Kimo2chTextBrowser::closePopup()
{
#if 0
	if(popupBrowser != NULL)
	{
		popupBrowser->closePopup();
	}
	if(objectName() == "Popup")
	{
//		parentWidget()->close();
		delete	popup;
	}
	popup = NULL;
#endif
	if(popup)
	{
		popup->close();
//		delete	popup;
		popup = NULL;
	}
//	popupBrowser = NULL;
}

void	Kimo2chTextBrowser::enterEvent(QEvent	*/*e*/)
{
	isEnterCursor = true;
}

void	Kimo2chTextBrowser::leaveEvent(QEvent	*/*e*/)
{
	if(parentPopup && isEnterCursor && cm == NULL)
	{
		parentPopup->closePopup();
	}
}

void	Kimo2chTextBrowser::setFontFamily(const	QString	&fontFamily)
{
	browserFontFamily = fontFamily;
	QTextBrowser::setFontFamily(fontFamily);
}

void	Kimo2chTextBrowser::setFontPointSize(qreal	s)
{
	browserFontPointSize = s;
	QTextBrowser::setFontPointSize(s);
}

/*******************************************************
			ParserThread
*******************************************************/

ParserThread::ParserThread(Kimo2chTextBrowser	*parent)
	: QThread(parent), browser(parent)
{
//	browser = parent;
}

void	ParserThread::run()
{
	datRef = browser->getDatRef();
	readedCount = browser->getReadedCount();
	QMultiMap<QString, int>	*idMap = browser->getIdMap();
	idMap->clear();
	QString	parsedText;
	QTextStream	parsedStream(&parsedText);
	// Font family and size settings
	parsedStream << "<body style=\"font-family : " << browser->browserFontFamily << "; font-size :" << browser->browserFontPointSize << "pt;\">";
	
	QStringList::iterator	iter = datRef->begin();
	int	i = 1;
	while(iter != datRef->end())
	{
		QString	datLine = *iter;
		QStringList	resItems = datLine.split("<>");
		if(resItems.count() < 4)
		{
//			QMessageBox::warning(browser, tr("File error"), tr("DAT file is broken. Res format invalid."));
			return;
		}
		QString	idAnchor;
		int	idstart = resItems[2].indexOf("ID:");
		if(idstart >= 0)
		{
			int idlength = resItems[2].indexOf(QRegExp("\\s"), idstart) - idstart;
			QString	id = resItems[2].mid(idstart, (idlength < 0) ? -1 : idlength);
			idMap->insert(id, i);
		}
		parsedStream << "<a name=\"res" << i << "\">" << i << "</a> " << ResParser::parse(datLine);
		iter++;
		if(i > 1 && i == readedCount && iter != datRef->end())
		{
			parsedStream << "<hr>" << tr("New message(s)") << "<hr>";
		}
		i++;
	}
	parsedStream << "<br>";	// Add last blank line for margin.
	browser->setParsedText(parsedText);
	parseCompleted();
}
