#ifndef MAINWINDOWIMPL_H
#define MAINWINDOWIMPL_H
//
#include <QtCore>
#include <QtNetwork>
#include <QtGui>
#include <QtXml>
#include "ui_mainwindow.h"
#include "src/globals.h"
#include "src/kimo2chtimerdialog.h"
#include "src/kimo2chprefsdialog.h"
#include "src/tabwidgetimpl.h"
#include "src/browsewidgetimpl.h"
#include "src/xmlprefshandler.h"
#include "src/kimo2chtextbrowser.h"
#include "src/abonedialog.h"
#include "src/filteredit.h"
#include "src/kimo2chfavoriteshandler.h"
#include "src/rescountitem.h"
#include "src/cp932codec.h"
//

// Preference items
const QString INITIALBOARDURL = "http://www.2ch.net/bbsmenu.html";

class MainWindowImpl : public QMainWindow, public Ui::MainWindow
{
Q_OBJECT
public:
	MainWindowImpl( QWidget * parent = 0, Qt::WFlags f = 0 );

	virtual	~MainWindowImpl();
	
	QList<Favorite>	*getFavoriteList()	{ return	favoriteList; }
	QNetworkProxy	*getProxy()	{ return	&globalProxy; }
	
	bool	isAutoReloading()	{ return	autoReloading; }
	
public slots:
	void	downReloadButton();
	void	upReloadButton();
	void	openLink(const QUrl	&url);
	void	startAutoReload(bool	state = true);
	
private slots:
	// View control
	void	showBoardList();
	void	showThreadList();
	void	showResEditTab();
	void	hideAllPanes();
	void	hidePopups();
	void	filterLabelChange();
	void	filterLabelRevert();
	void	searchLabelChange();
	void	searchLabelRevert();
	// Preferences dialog
	void	showPrefsPanel();
	void	prefsChange();
	// Reload timer dialog
	void	showTimerDialog();
	void	timerChange();
	// Favorite list update dialog
	void	showUpdateDialog();
	void	startAutoUpdate(bool	state = true);
	// Search dialog
	void	doSearch(QString	searchText = 0);
	void	doSearchBackward();
	void	doFilter(QString	filterText);
	// Abone dialog
	void	showAboneDialog();
	void	aboneChange();
	void	closeAboneDialog();
	// Buttons
	void	reject();
	void	post();
	void	reload();
	void	closeTab(int	index);
	void	loadSubject();
	void	toggleAddFavoriteButton();
	// Networking
	void	rebuildBoardList();
	void	boardSelected(const QModelIndex index);
	void	findNextThreads();
	void	threadTabChanged(int index);
	
	void	readThread(const QModelIndex index);
	void	openTab(QString	url);
	void	clearCache();
	
	void	networkRequestFinished(int id, bool error);
	void	httpResponse(const QHttpResponseHeader &header);
	void	postResponseHeader(const QHttpResponseHeader	&header);
	void	postRequestFinished(int	postReqId, bool	error);
	
	void	updateFavoriteList();
private:
	// Prefs
	void	readPrefs();
	void	writePrefs();
	void	readFavorites();
	void	writeFavorites();
	void	buildBoardList();			// Build boards list
	
	void	setupProxy();
	// Session
	void	readSession();
	void	writeSession();
	
	// New editor tab
	QWidget*	createNewEditorTab(QWidget *parent);
	
	// Thread view column width restore
	void	rememberColumnWidth();
	void	restoreColumnWidth();
	
	// Utilities
	QString	urlToDat(QString	url);
	QString	datToUrl(QString	dat);
	
// Data members
private:
	// Menus
	QMenu	*reloadMenu;
	QMenu	*updateMenu;
	
	// Dialogs
	Kimo2chPrefsDialog	*prefsDialog;	// Preferences dialog
	Kimo2chTimerDialog	*timerDialog;	// Auto reload timer dialog
	QAction	*reloadStart;
	QAction	*updateStart;
	AboneDialog			*aboneDialog;	// Name, mail or ID abone dialog
	
	// Prefs data
	Kimo2chPrefs	*prefs;				// Preferences data
	struct	aboneData	*aboneData;		// A-bone data
	QStringList	*aboneKeywordList;		// A-bone keyword for hiding
	QDir	*storageDir;
	
	// Timer
	QTimer	reloadTimer;
	QTimer	updateTimer;
	int	timerSec;
	// Networking
	QHttp	*network;
	QHttp	*favoriteNetwork;
	QFile	*boardFile;
	QBuffer *subjectBuf;
	bool	isFavoritesUpdating;
	
	QNetworkProxy	globalProxy;
	
	int	boardListGetID;
	int subjectGetID;
	int	favoriteUpdateID;
	
	QHttp	*postConnection;
	QBuffer *postResponse;
	QString	postServer;
	int postReqId;
	
	QString	currentBoard;
	QString	currentBoardName;
	
	QString	currentLoadingBoard;
	// Containers
	QStandardItemModel	*boardModel;
	QStandardItemModel	*threadModel;
	QStandardItemModel	*oldThreadModel;
	QStandardItemModel	*filteredModel;
	QList<Favorite>	*favoriteList;
	
	// Temporary containers
	QList<QList<Favorite>* >	targetBoardList;
	
	// Favorite status
	int	unreadThreadNum;
	int	unreadResNum;
	
	// Others
	bool	isReadOnNewTab;
	bool	initialTab;
	
	// Auto update flags
	bool	autoReloading;
	bool	isAutoUpdating;
};

#endif
