#include "resparser.h"

QMap<QString, bool>	ResParser::aboneName;
QMap<QString, bool>	ResParser::aboneMail;
QMap<QString, bool>	ResParser::aboneId;
QMap<QString, bool> ResParser::aboneWord;

/*************************************
	ResParser class
*************************************/
/*
	Parse dat file
*/
QString	ResParser::parse(const QString	resLine)
{
	QString	parsedText;
	QTextStream	parsed(&parsedText);
	
	if(!resLine.contains("<>"))
	{
		return NULL;
	}
	QStringList	resItems = resLine.split("<>");
	if(resItems.count() < 4)
	{
		return NULL;
	}
	// Pickup ID and date field from header
	int	idstart = resItems[2].indexOf("ID:");
	QString	dateField = resItems[2], idtext;
	if(idstart >= 0)
	{
		int	idend = resItems[2].indexOf(QRegExp("(\\s)"), idstart);
		idend = (idend == -1) ? resItems[2].length() : idend;
		idtext = resItems[2].mid(idstart, idend - idstart);
		dateField.insert(idend, "</a>");
		dateField.insert(idstart, QString("<a href=\"#%1\">").arg(idtext));
	}
	// Name abone
	if(aboneName.contains(resItems[0]))
	{
		parsed << " <a style=\"color:green;text-decoration:none;\">" << ABONETEXT << "(" << resItems[0] << ")" << "</a> [" << ABONETEXT << "] "
			<< resItems[2] << "<br><p></p>";
		return parsedText;
	}
	// Mail abone
	if(aboneMail.contains(resItems[1]))
	{
		parsed << " <a style=\"color:green:text-decoration:none;\">" << ABONETEXT << "</a> [" << ABONETEXT << " (" << resItems[1] << ") ] "
			<< resItems[2] << "<br><p></p>";
		return parsedText;
	}
	// ID abone
	if(aboneId.contains(idtext))
	{
		parsed << " <a href=\"\" style=\"color:green\">" << ABONETEXT << "</a> [" << ABONETEXT << "] " << dateField << "ID:" << ABONETEXT << "<p></p>\n";
		return parsedText;
	}
	// Word abone
	QStringList	aboneWordList = aboneWord.keys(false);
	QStringList::iterator	iter = aboneWordList.begin();
	while(iter != aboneWordList.end())
	{
		if(resItems[3].contains(*iter))
		{
			parsed << " <a style=\"color:green;text-decoration:none;\">" << ABONETEXT << "</a> [" << ABONETEXT 
				<< "] " << dateField << " ID:" << ABONETEXT << "<p>" << ABONETEXT << "( " << *iter << " )</p>\n";
			return parsedText;
		}
		iter++;
	}
	// Normal message
	parsed << "<a style=\"color:green;text-decoration:none;\" href=\"#NAME:" << resItems[0] << "\">" << resItems[0] << "</a>"
		<< "<a style=\"color:black;text-decoration:none;\" href=\"#MAIL:" << resItems[1] << "\">[" << resItems[1] << "</a>] " << dateField 
		<< "\n<p style=\"white-space:pre\">" << rewriteAnchor(resItems[3]) << "</p>\n";
		
	return	parsedText;
}

// Rewrite anchors in message
QString	ResParser::rewriteAnchor(const QString	originalText)
{
	QString	tempText(originalText);
	tempText.replace("‾", "~");
	tempText.replace("<br>", "\n");
	// Res number anchor rewrite
	tempText.remove(QRegExp("<a \\S+ target=\"_blank\">"));	// Remove start tags
	int	pos = 0;
	while((pos = tempText.indexOf(QRegExp("&gt;&gt;(\\,|\\-|\\d)*</a>"), pos)) != -1)
	{
		int	endpos = tempText.indexOf("</a>", pos);
		QString	anchor = tempText.mid(pos + 8, endpos - pos - 8);
		tempText.insert(pos, QString("<a href=\"#res%1\">").arg(anchor));
		pos = endpos + anchor.length() + 15;
	}
	// Anchor insert to URLs
	pos = 0;
	while((pos = tempText.indexOf(QRegExp("(http|ttp|tp)://"), pos)) != -1)
	{
		int	endpos = tempText.indexOf(QRegExp("([^a-zA-Z0-9~%&#=\\(\\)\\:/\\-\\.\\,\\?\\_])"), pos);
		tempText.insert(endpos, "</a>");
		QString	url = tempText.mid(pos, endpos - pos);
		url = url.mid(url.indexOf(":"));
		tempText.insert(pos, QString("<a href=\"http%1\">").arg(url));
		pos = endpos + url.length() + 15;
	}
	return	tempText;
}

/*
	Abone file
*/
// Load abone file
void	ResParser::loadAboneFile(QFile	*aboneFile)
{
	QXmlSimpleReader	reader;
	QXmlInputSource	source(aboneFile);
	AboneFileHandler	handler;// = new AboneFileHandler;
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);
	reader.parse(&source);
}

// Save abone file
void	ResParser::saveAboneFile(QFile	*aboneFile)
{
	if(!aboneFile->open(QIODevice::WriteOnly))
	{
		return;
	}
	qDebug() << "Write to Abone file";
	QXmlStreamWriter	*writer = new QXmlStreamWriter(aboneFile);
	writer->setAutoFormatting(true);
	writer->writeStartDocument();
	writer->writeStartElement("", "AboneLists");
	// Save abone lists 
	QMap<QString, bool>::const_iterator	iter;
	// Write abone ID list
	if(!(aboneId.isEmpty()))
	{
		writer->writeStartElement("", "AboneIdList");
		iter = aboneId.constBegin();
		while(iter != aboneId.constEnd())
		{
			writer->writeStartElement("", "AboneItem");
			writer->writeAttribute("", "Hilite", iter.value() ? "true" : "false");
			writer->writeCharacters(iter.key());
			writer->writeEndElement();
			iter++;
		}
		writer->writeEndElement();
	}
	// Write abone name list
	if(!(aboneName.isEmpty()))
	{
		writer->writeStartElement("", "AboneNameList");
		iter = aboneName.constBegin();
		while(iter != aboneName.constEnd())
		{
			writer->writeStartElement("", "AboneItem");
			writer->writeAttribute("", "Hilite", iter.value() ? "true" : "false");
			writer->writeCharacters(iter.key());
			writer->writeEndElement();
			iter++;
		}
		writer->writeEndElement();
	}
	// Write abone mail list
	if(!(aboneMail.isEmpty()))
	{
		writer->writeStartElement("", "AboneMailList");
		iter = aboneMail.constBegin();
		while(iter != aboneMail.constEnd())
		{
			writer->writeStartElement("", "AboneItem");
			writer->writeAttribute("", "Hilite", iter.value() ? "true" : "false");
			writer->writeCharacters(iter.key());
			writer->writeEndElement();
			iter++;
		}
		writer->writeEndElement();
	}
	// Write abone word list
	if(!(aboneWord.isEmpty()))
	{
		writer->writeStartElement("", "AboneWordList");
		iter = aboneWord.constBegin();
		while(iter != aboneWord.constEnd())
		{
			writer->writeStartElement("", "AboneItem");
			writer->writeAttribute("", "Hilite", iter.value() ? "true" : "false");
			writer->writeCharacters(iter.key());
			writer->writeEndElement();
			iter++;
		}
		writer->writeEndElement();
	}
	writer->writeEndElement();
	writer->writeEndDocument();
	delete	writer;
	aboneFile->close();
}

/***************************************
	AboneFileHandler class
***************************************/
AboneFileHandler::AboneFileHandler()
{
	ResParser::aboneName.empty();
	ResParser::aboneMail.empty();
	ResParser::aboneId.empty();
}

AboneFileHandler::~AboneFileHandler()
{
	
}

bool	AboneFileHandler::startDocument()
{
	return	true;
}
bool	AboneFileHandler::endDocument()
{
	return	true;
}

bool	AboneFileHandler::startElement(const QString	&/*namespaceURI*/, const QString	&/*localName*/, const QString	&qName, const QXmlAttributes	&atts)
{
	if(qName == "AboneNameList")
	{
		targetList = &(ResParser::aboneName);
		return	true;
	}
	if(qName == "AboneMailList")
	{
		targetList = &(ResParser::aboneMail);
		return	true;
	}
	if(qName == "AboneIdList")
	{
		targetList = &(ResParser::aboneId);
		return	true;
	}
	if(qName == "AboneWordList")
	{
		targetList = &(ResParser::aboneWord);
		return	true;
	}
	if(qName == "AboneItem")
	{
		if(atts.value("Hilite") == "true")
			itemState = true;
		else
			itemState = false;
		return	true;
	}
	return	true;
}

bool	AboneFileHandler::endElement(const QString	&/*namespaceURI*/, const QString	&/*localName*/, const QString	&qName)
{
	if(qName == "AboneNameList" || qName == "AboneMailList" || qName == "AboneIdList" || qName == "AboneWordList")
		targetList = NULL;
	return	true;
}

bool	AboneFileHandler::characters(const QString	&ch)
{
	if(targetList == NULL)
		return	true;
	if(ch.contains(QRegExp("\\S")))
	targetList->insert(ch, itemState);
	return	true;
}
// End of file